import { Component, Prop } from 'vue-property-decorator';
import { ZdComponentRender } from '@zeedhi/vuetify';
import { moment, DialogService } from '@zeedhi/common';
import {
  Loader, Http, IDictionary,
  I18n,
} from '@zeedhi/core';
import { Cardapio as Info } from './common/Cardapio';

@Component({})
export default class Cardapio extends ZdComponentRender {
    @Prop({ type: Array, default: () => [] }) public cardData!: IDictionary;
    
    @Prop({ type: Array, default: () => [] }) public dates!: IDictionary;
    
    @Prop({ type: Array, default: () => [] }) public menuContent!: IDictionary;
    
    @Prop({ type: Object, default: () => ({}) }) public filterData!: IDictionary;
    
    @Prop({ type: Array, default: () => [] }) public colorCard!: IDictionary;
    
    @Prop({ type: Number, default: 3 }) public preview!: number;
    
    public instance!: Info;
    
    public utilities = Loader.getInstance('Utilities');
    
    public instanceType: typeof Info = Info;
    
    private dadosDoPrato = Loader.getInstance('Pla00115DadosDoPrato');
    
    private modalComensais = Loader.getInstance('Pme00600ComensaisEstimadoCardapio');
    
    private modalExclude = Loader.getInstance('Pme00600ExclusaoPrato');
    
    private keyPressFunctions: IDictionary = {};

    private keyboardEnaled = false;
    
    private controlActived = false;
    
    private processDate = '';

    private currentSelectedRegister: IDictionary = {};
    
    private currentCopiedRegister: IDictionary = {};
    
    private actionEnabled = false;

    public created() {
      this.instance.bindNewGridAction = this.bindNewGridAction;
      this.instance.moveTableSelection = this.moveTableSelection;
      this.instance.getCurrentSelectedRegister = this.getCurrentSelectedRegister;
      this.instance.setMenuContent = this.setMenuContent;
      this.instance.forceUpdateInCard = this.forceUpdateInCard;
      this.instance.updateComens = this.updateComens;
      this.instance.dateLessDateOfProcess = this.dateLessDateOfProcess;
      this.instance.setValueAction = this.setValueAction;
      
      this.currentSelectedRegister.x = null;
      this.currentSelectedRegister.y = null;
    }

    public destroyed() {
      this.stopKeyboardHandler();
    }

    public mounted() {
      this.makeHorizontalDefault();
      this.startKeyboardHandler();
      this.startProcessDate();
    }
    
    public update() {
      this.makeHorizontalDefault();
    }
    
    private startProcessDate() {
      this.utilities.getProcessDate().then((date: string) => {
        const format = 'DD/MM/YYYY';
        this.processDate = moment(date, format).format(format);
      });
    }
    
    private validDateBloqCard(dtIni: string, dtFin: string) {
      const format = 'DD/MM/YYYY';
      const dataIni = moment(dtIni, format);
      const dataFin = moment(dtFin, format);
      if (dataIni < dataFin) return true;
      return false;
    }
    
    public dateLessDateOfProcess(date: string): boolean {
      return this.validDateBloqCard(this.processDate, date);
    }
    
    // Se a data do cardapio for menor que a data de processamento, colore o backgroud do container de vermelho claro.
    public isBlockDate(dataCardapio: string) {
      if (this.dateLessDateOfProcess(dataCardapio)) return '';
      return 'background-color: #ff000061';
    }
    
    public isSpecific(idcardespec: string) {
      if (this.instance.filterData.aggregated !== null && idcardespec === 'S') {
        if (this.instance.colorCard.length > 0) {
          let color = 'color: #';
          const corCard = this.instance.colorCard[0].NRCORCARDESP;
          color += corCard.toString(16);
          
          return color;
        }
      }
      
      return '';
    }
  
    public calcGrid(size: number) {
      if (size) {
        let gtc = 'grid-gap: 2px; grid-template-columns: ';
        const cols = size;

        for (let i = 0; i < cols; i += 1) {
          gtc += '200px ';
        }
        return gtc;
      }
      return '';
    }
    
    public showValueInNumber(plate: IDictionary) {
      const objectDate = this.instance.dates.find((data: IDictionary) => data.DTCARDIAPME === plate.DTCARDIAPME);
      return ((plate.VRPECONSCPME / 100) * objectDate.QTCOMESTCPME);
    }
    
    public formatDate(date: string) {
      const dateMonth = moment(date, 'DD/MM/YYYY').format(this.utilities.getFormatLocaleWithWeek());
      return dateMonth;
    }
    
    public formatComens(comens: IDictionary) {
      if (comens.QTCOMESTCPME === undefined || comens.QTCOMESTCPME === null) comens.QTCOMESTCPME = 0;
      return comens.QTCOMESTCPME;
    }
    
    public openPlateDataModal(plate: IDictionary) {
      if (this.dateLessDateOfProcess(plate.DTCARDIAPME)) {
        let row: IDictionary = {};
        row = this.mountDataInsertCardia(plate);
        this.insertIntoCardia(row).then((response) => {
          const comensais = response.QTCOMESTCPME;
          const dtcardia = response.DTCARDIAPME;
          if (comensais !== null && comensais !== 0) {
            this.updateComens(comensais, dtcardia);
            plate.QTCOMESTCPME = comensais;
            this.setValueAction(true);
            this.dadosDoPrato.showModal(plate);
          } else {
            this.utilities.showAlert('dinerNull', 'pme::comensalNulo', 'error');
          }
        });
      } else {
        this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
      }
    }
    
    private mountDataInsertCardia(plate: IDictionary): IDictionary {
      plate.filterData = this.instance.filterData;
      const idcard = this.instance.dates.filter((data: IDictionary) => data.DTCARDIAPME === plate.DTCARDIAPME);
      plate.IDCARDESPEC = (idcard[0].IDCARDESPEC === undefined) ? 'N' : idcard[0].IDCARDESPEC;
      plate.QTCOMESTCPME = idcard[0].QTCOMESTCPME;
      const row = plate;
      row.WEEKDAY = moment(plate.DTCARDIAPME, 'DD/MM/YYYY').day();
      
      return row;
    }
    
    public insertIntoCardia(row: IDictionary) {
      return Http.post('/getCardia/get', { requestType: 'Row', row }).then((response) => {
        const responseDataCardia = response.data.dataset.cardiaOnClickPlate[0];
        return responseDataCardia;
      });
    }
    
    public changeTypeView(typeView: string) {
      this.instance.filterData.IDPERCCOME = (typeView == 'C') ? '%' : 'C'; // eslint-disable-line
      localStorage.setItem('IDPERCCOME', this.instance.filterData.IDPERCCOME);
      this.forceUpdateInCard();
    }
    
    public updateComens(comens: number, date: string) {
      this.instance.dates.forEach((data: IDictionary) => {
        if (data.DTCARDIAPME === date) {
          data.QTCOMESTCPME = comens;
        }
      });
      this.forceUpdateInCard();
    }
    
    public openModalComen(comensal: IDictionary) {
      const row = comensal;
      row.filterData = this.instance.filterData;
      Http.post('/updateComen', { requestType: 'Row', row }).then((response: IDictionary) => {
        if (response.data.dataset.hasComen.length > 0) {
          comensal.QTCOMESTCPME = (comensal.QTCOMESTCPME === 0) ? response.data.dataset.hasComen[0].QTCOMESTCPME : comensal.QTCOMESTCPME;
          this.forceUpdateInCard();
          this.setValueAction(true);
          this.modalComensais.showModal(comensal);
        } else {
          this.utilities.showAlert('errorInsert', 'pme::errorInsercaoComensal', 'error');
        }
      });
    }
    
    private makeHorizontalDefault() {
      const platesContainer: HTMLElement = document.getElementById('platesContainer') as HTMLElement;
      if (platesContainer && platesContainer.addEventListener) {
        window.addEventListener('keydown', (e) => {
          if (this.currentSelectedRegister?.CDTIPOPRAT) {
            setTimeout(() => {
              if (e.key === 'ArrowLeft' || e.key === 'ArrowRight') {
                const delta = e.key === 'ArrowLeft' ? 1 : -1;
                const plateContainer: HTMLElement = document.getElementById('platesContainer') as HTMLElement;
                const titulosPratos: HTMLElement = document.getElementById('titulosPratos') as HTMLElement;
                plateContainer.scrollLeft -= delta * 192; // * by 40
                titulosPratos.scrollLeft -= delta * 192; // * by 40
                if (this.instance.preview === 2) {
                  const comensais: HTMLElement = document.getElementById('comensais') as HTMLElement;
                  comensais.scrollLeft -= delta * 192;
                }
                e.preventDefault();
              }
              return true;
            }, 1);
          }
        });
      }
    }
    
    // private moveHorizontalAction(plateIdx: number) {
    //   const titulosPratos: HTMLElement = document.getElementById('titulosPratos') as HTMLElement;
    //   if (titulosPratos.childNodes[0].childNodes.length > 0) {
    //     (titulosPratos.childNodes[0].childNodes[plateIdx] as HTMLElement).scrollIntoView();
    //   }
    // }

    public changePlateSelection(menuContentIdx: number, plateIdx: number, selection?: boolean) {
      if (plateIdx > -1) {
        const selectedPlate = this.instance.menuContent[menuContentIdx][plateIdx];
        let newSelection = selection;
        if (selectedPlate !== undefined) {
          if (newSelection === undefined) {
            newSelection = !selectedPlate.isSelected;
          }
  
          this.instance.menuContent[menuContentIdx].splice(plateIdx, 1, Object.assign(selectedPlate, { isSelected: newSelection }));
          this.keyboardEnaled = true;
  
          if (newSelection) {
            this.currentSelectedRegister = this.instance.menuContent[menuContentIdx][plateIdx];
            this.currentSelectedRegister.x = menuContentIdx;
            this.currentSelectedRegister.y = plateIdx;
            // this.moveHorizontalAction(plateIdx);
          }
        }
      } else {
        this.changeDinersSelection(menuContentIdx, selection);
      }
    }

    private changeDinersSelection(dinerIndex: number, selection?: boolean) {
      const selectedDiner = this.instance.dates[dinerIndex];
      let newSelection = selection;
      
      if (newSelection === undefined) {
        newSelection = !selectedDiner.isSelected;
      }

      this.instance.dates.splice(dinerIndex, 1, Object.assign(selectedDiner, { isSelected: newSelection }));
      this.keyboardEnaled = true;

      if (newSelection) {
        this.currentSelectedRegister = this.instance.dates[dinerIndex];
        this.currentSelectedRegister.x = dinerIndex;
        this.currentSelectedRegister.y = -1;
      }
    }

    public onPlateClick(plate: object, menuContentIdx: number, plateIdx: number) {
      const currentX = this.currentSelectedRegister.x;
      const currentY = this.currentSelectedRegister.y;

      if (currentX !== null && currentY !== null && !(currentX === menuContentIdx && currentY === plateIdx)) {
        this.changePlateSelection(this.currentSelectedRegister.x, this.currentSelectedRegister.y, false);
      }
      this.changePlateSelection(menuContentIdx, plateIdx);
    }
    
    /* Caso precisar verificar os keydown
    * https://www.w3.org/2002/09/tests/keys-cancel2.html?
    * https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/keyCode
    */
    private startKeyboardHandler() {
      window.addEventListener('keydown', this.keyboardHandler, false);
      
      this.bindNewGridAction(37, () => this.moveTableSelection('LEFT'));
      this.bindNewGridAction(38, () => this.moveTableSelection('UP'));
      this.bindNewGridAction(39, () => this.moveTableSelection('RIGHT'));
      this.bindNewGridAction(40, () => this.moveTableSelection('DOWN'));
      this.bindNewGridAction(13, () => this.enterKeyboardAction());
      this.bindNewGridAction(115, () => this.excludeRegisterAction());
      this.bindNewGridAction(67, () => this.copyCurrentRegisterAction());
      this.bindNewGridAction(86, () => this.pasteCopiedRegisterAction());
      this.bindNewGridAction(88, () => this.cutRegisterAction());
    }
    
    private cutRegisterAction() {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (this.currentSelectedRegister.CDPRATO !== undefined) {
            if (this.currentSelectedRegister.CDPRATO !== null) {
              if (this.dateLessDateOfProcess(this.currentSelectedRegister.DTCARDIAPME)) {
                this.copyRegister(JSON.parse(JSON.stringify(this.currentSelectedRegister)));
                this.excludeRegisterAction();
                this.pasteCopiedRegisterAction();
              } else {
                this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
              }
            } else {
              this.utilities.showAlert('erroCopy', 'pme::naoPossuiRegNoLocal', 'error');
            }
          } else {
            this.utilities.showAlert('errorAction', 'pme::comensaisSelecionado', 'error');
          }
        }
      }
    }
    
    private pasteCopiedRegisterAction() {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (Object.keys(this.currentCopiedRegister).length > 0) {
            if (this.currentSelectedRegister.CDPRATO !== undefined) {
              const currentDataSave = this.currentSelectedRegister;
              this.insertRegisterCopiedInCardia(currentDataSave);
            } else {
              this.utilities.showAlert('errorAction', 'pme::comensaisSelecionado', 'error');
            }
          } else {
            this.utilities.showAlert('erroPaste', 'pme::naoPossuiRegCopiado', 'error');
          }
        }
      }
    }
    
    private insertRegisterCopiedInCardia(currentDataSave: IDictionary) {
      let row: IDictionary = {};
      row = this.mountDataInsertCardia(currentDataSave);
      this.insertIntoCardia(row).then((response) => {
        const comensais = response.QTCOMESTCPME;
        const dtcardia = response.DTCARDIAPME;
        if (comensais !== null && comensais !== 0) {
          this.updateComens(comensais, dtcardia);
          currentDataSave.QTCOMESTCPME = comensais;
          const rowSave = this.mountDataSavePlatePaste(currentDataSave);
          this.saveRegisterCopied(rowSave);
        } else {
          this.utilities.showAlert('dinerNull', 'pme::comensalNulo', 'error');
        }
      });
    }
    
    private saveRegisterCopied(row: IDictionary) {
      this.dadosDoPrato.callSavePlateData(row).then((response: boolean) => {
        if (response) {
          this.currentSelectedRegister.CDPRATO = this.currentCopiedRegister.CDPRATO;
          this.currentSelectedRegister.NMPRATO = this.currentCopiedRegister.NMPRATO;
          this.currentSelectedRegister.CDFILAUXPRAT = this.currentCopiedRegister.CDFILAUXPRAT;
          this.forceUpdateInCard();
        }
      });
    }
    
    private mountDataSavePlatePaste(currentDataSave: IDictionary): IDictionary {
      const row: IDictionary = {};
      currentDataSave.VRPECONSCPME = this.currentCopiedRegister.VRPECONSCPME;
      row.plateData = [[currentDataSave]];
      row.selectedPlate = this.currentCopiedRegister;
      row.numberDiners = currentDataSave.QTCOMESTCPME;
      
      return row;
    }
    
    private copyCurrentRegisterAction() {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (this.currentSelectedRegister.CDPRATO !== null) this.copyRegister(this.currentSelectedRegister); // == null, ele existe no objeto mas n possui dados
          else this.utilities.showAlert('erroCopy', 'pme::naoPossuiRegNoLocal', 'error');
        }
      }
    }
    
    private copyRegister(register: IDictionary) {
      if (register.CDPRATO !== undefined) {
        this.currentCopiedRegister = register;
      } else {
        this.utilities.showAlert('errorAction', 'pme::comensaisSelecionado', 'error');
      }
    }
    
    private excludeRegisterAction() {
      if (this.checkActionDisabled()) {
        if (this.currentSelectedRegister.CDPRATO !== undefined) {
          if (this.currentSelectedRegister.CDPRATO !== null) {
            if (this.dateLessDateOfProcess(this.currentSelectedRegister.DTCARDIAPME)) {
              this.setValueAction(true);
              DialogService.show({
                name: 'excludeRegisterAction',
                type: 'warning',
                title: 'pme::atencao',
                text: I18n.translate('pme::excluirPrato', { PARAM1: this.currentSelectedRegister.NMPRATO }),
                buttons: [
                  {
                    name: 'yes',
                    component: 'ZdButton',
                    label: 'pme::sim',
                    events: { click: () => this.excludeRegister() },
                    outline: true,
                  },
                  {
                    name: 'no',
                    component: 'ZdButton',
                    label: 'pme::nao',
                    events: { click: () => this.hideDialogService() },
                  },
                ],
              });
            } else {
              this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
            }
          } else {
            this.utilities.showAlert('errorInsert', 'pme::naoPossuiRegNoLocal', 'error');
          }
        } else {
          this.utilities.showAlert('errorAction', 'pme::comensaisSelecionado', 'error');
        }
      }
    }
    
    private hideDialogService() {
      this.setValueAction(false);
      DialogService.hide();
    }
    
    private excludeRegister() {
      this.modalExclude.callExcludeRoute(this.rowExcludeKeyboard()).then(() => {
        let arrUpdate: IDictionary = [];
        this.instance.menuContent.forEach((cel: IDictionary) => {
          arrUpdate = cel.filter((plates: IDictionary) => plates.DTCARDIAPME === this.currentSelectedRegister.DTCARDIAPME);
          if (arrUpdate.length > 0) {
            arrUpdate = arrUpdate.filter((plate: IDictionary) => plate.CDPRATO !== null);
            if (arrUpdate.length <= 1) this.updateComens(0, this.currentSelectedRegister.DTCARDIAPME);
          }
        });
        this.currentSelectedRegister.CDPRATO = null;
        this.currentSelectedRegister.NMPRATO = null;
        this.currentSelectedRegister.CDFILAUXPRAT = null;
        this.currentSelectedRegister.NRCARDIAPME = null;
        this.currentSelectedRegister.VRPECONSCPME = null;
        this.hideDialogService();
      });
    }
    
    private rowExcludeKeyboard(): IDictionary {
      const row: IDictionary = {};
      row.initialDateExclude = this.currentSelectedRegister.DTCARDIAPME;
      row.finalDateExclude = this.currentSelectedRegister.DTCARDIAPME;
      row.unity = (this.instance.filterData.aggregated !== null) ? 'E' : 'C';
      row.selectSpecific = (this.instance.filterData.aggregated !== null) ? [this.instance.filterData.aggregated.CDFILIAL] : [];
      row.platesSelect = [`${this.currentSelectedRegister.CDTIPOPRAT} - ${this.currentSelectedRegister.NRSEQCNFCPME}`];
      row.filterData = this.instance.filterData;
      return row;
    }
    
    private enterKeyboardAction() {
      if (this.checkActionDisabled()) {
        if (Object.keys(this.currentSelectedRegister).length > 0) {
          if (this.currentSelectedRegister.CDTIPOPRAT !== undefined) this.openPlateDataModal(this.currentSelectedRegister);
          else this.openModalComen(this.currentSelectedRegister);
        }
      }
    }

    private stopKeyboardHandler() {
      window.removeEventListener('keydown', this.keyboardHandler);
    }
    
    private activeControlHandler(e: KeyboardEvent) {
      this.controlActived = e.ctrlKey;
    }
    
    private keyboardHandler(e: KeyboardEvent) {
      this.activeControlHandler(e);
      if (Array.isArray(this.keyPressFunctions[`key_${e.keyCode}`]) && this.keyboardEnaled) {
        this.keyPressFunctions[`key_${e.keyCode}`].forEach((action: (e: KeyboardEvent) => void) => {
          if ([32, 37, 38, 39, 40].indexOf(e.keyCode) > -1) {
            e.preventDefault();
          }
          action(e);
        });
      }
    }

    public bindNewGridAction(charKey: number, action: object) {
      if (this.keyPressFunctions[`key_${charKey}`] === undefined) {
        this.keyPressFunctions[`key_${charKey}`] = [];
      }
      this.keyPressFunctions[`key_${charKey}`].push(action);
    }

    public moveTableSelection(direction: string) {
      if (this.checkActionDisabled()) {
        const toMovePlate = Object.assign(this.currentSelectedRegister);
        const selectedElement = document.querySelector('.card-anal-container .selected') as HTMLElement;
  
        this.changePlateSelection(toMovePlate.x, toMovePlate.y);
  
        switch (direction) {
          case 'UP':
            if (this.instance.preview === 3) {
              if (toMovePlate.x > 0) toMovePlate.x -= 1;
            } else if (this.instance.preview === 2) {
              if (toMovePlate.y > 0) toMovePlate.y -= 1;
              if (toMovePlate.y === -1) toMovePlate.y = this.instance.menuContent[toMovePlate.x].length - 1;
            } else if (this.instance.preview === 1) {
              const auxMove = toMovePlate.x;
              if (toMovePlate.x !== 0) {
                if (toMovePlate.y === 0) {
                  toMovePlate.x -= 1;
                  toMovePlate.y = -1;
                }
              }
              
              if (toMovePlate.y > 0) toMovePlate.y -= 1;
              if (toMovePlate.y === -1 && auxMove === toMovePlate.x) toMovePlate.y = this.instance.menuContent[toMovePlate.x].length - 1;
            }
            break;
          case 'DOWN':
            if (this.instance.preview === 3) {
              if (toMovePlate.x < this.instance.menuContent.length - 1) toMovePlate.x += 1;
            } else if (this.instance.preview === 2) {
              toMovePlate.y += 1;
              if (toMovePlate.y >= this.instance.menuContent[toMovePlate.x].length) toMovePlate.y = -1;
            } else if (this.instance.preview === 1) {
              if (toMovePlate.y === -1) toMovePlate.x += 1;
              if (toMovePlate.x < this.instance.menuContent.length) {
                toMovePlate.y += 1;
                if (toMovePlate.y >= this.instance.menuContent[toMovePlate.x].length) toMovePlate.y = -1;
              } else {
                toMovePlate.x -= 1;
              }
            }
            break;
          case 'LEFT':
            if (this.instance.preview === 3) {
              if (toMovePlate.y > -1) toMovePlate.y -= 1;
            } else if (this.instance.preview === 2) {
              if (toMovePlate.x !== 0) toMovePlate.x -= 1;
            }
            break;
          case 'RIGHT':
            if (this.instance.preview === 3) {
              if (toMovePlate.y < this.instance.menuContent[toMovePlate.x].length - 1) toMovePlate.y += 1;
            } else if (this.instance.preview === 2) {
              if (toMovePlate.x < this.instance.menuContent.length - 1) toMovePlate.x += 1;
            }
            break;
          default:
            break;
        }
  
        this.changePlateSelection(toMovePlate.x, toMovePlate.y);
  
        if (selectedElement !== undefined && selectedElement !== null) {
          selectedElement.scrollIntoView({
            block: 'center',
          });
        }
      }
    }

    public getCurrentSelectedRegister(): IDictionary {
      let currRegister;

      if (this.currentSelectedRegister.y === -1) {
        currRegister = this.instance.dates[this.currentSelectedRegister.x];
      } else if (this.currentSelectedRegister.y > -1) {
        currRegister = this.instance.menuContent[this.currentSelectedRegister.x][this.currentSelectedRegister.y];
      }

      return currRegister;
    }

    public setMenuContent(menuContent: IDictionary) {
      this.instance.menuContent = menuContent;
      this.focusCurrentSelectedRegister();
      this.changePlateSelection(0, 0);
    }
    
    public forceUpdateInCard() {
      this.$forceUpdate();
    }
    
    public focusCurrentSelectedRegister() {
      if (document.getElementsByClassName('container-max selected')[0] !== undefined) (document.getElementsByClassName('container-max selected')[0] as HTMLElement).focus();
    }
    
    // Verifica se tem alguma ação aberta no cardapio.
    private checkActionDisabled(): boolean {
      if (this.actionEnabled) return false;
      return true;
    }
    
    // Função utilizada para travar as ações do cardapio. OBS: precisa ta true para travar
    public setValueAction(actived: boolean) {
      this.actionEnabled = actived;
    }
    
    public changePreview(previewSelected: number) {
      this.instance.preview = previewSelected;
      this.forceUpdateInCard();
    }
    
    public calcContainer() {
      const divCalc = document.getElementsByClassName('v-expansion-panel-header');
      return `margin: ${(divCalc[0] as IDictionary)?.offsetHeight + 13}px auto 0`;
    }
}
