import {
  Http, Loader, Metadata, I18n, Datasource, IEventParam, IDictionary,
} from '@zeedhi/core';

import {
  Form, Button, DialogService, GridEditable, Grid,
} from '@zeedhi/common';
  
export class Ger96000CentralController {
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditableController = Loader.getInstance('GridEditableController');

  public save({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formCentral');
    
    if (form.validate()) {
      const grid: GridEditable = Metadata.getInstance('CENTRAL');
      const editedRows = grid.getEditedRows();
      
      editedRows.forEach((row: IDictionary) => {
        if (!row.__is_new) {
          row.__is_new = false;
        }
      });

      Http.post('/salvaCentral', { editedRows }).then((response: IDictionary) => {
        let param;
        if (response.data.length === 0) {
          grid.datasource.get();
          
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          this.gridEditableController.cancelEditing({ component });
        } else {
          const mensagem = response.data.dataset.mensagem[0];
          
          if (mensagem.indexOf('ERROR_E_AGREGADA') === 0) {
            param = mensagem.split(':');
            this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_E_AGREGADA', { PARAM1: param[1] }), 'error');
          } else if (mensagem.indexOf('ERROR_JA_CADASTRADA') === 0) {
            param = mensagem.split(':');
            this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_UNID_CENTRAL', { PARAM1: param[1] }), 'error');
          } else {
            this.utilities.showAlert('alertError', mensagem, 'error');
          }
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      });
    }
  }

  public carregaCentrais() {
    const central: GridEditable = Metadata.getInstance('CENTRAL');
    central.datasource.get().then(() => {
      central.datasource.currentRow = central.datasource.data[0];
    });
  }
  
  public salvaAgregada({ component }: Button) {
    let param;
    const form = Metadata.getInstance<Form>('formAgregada');
    if (form.validate()) {
      const grid: GridEditable = Metadata.getInstance('AGREGADA');
      const editedRows = grid.getEditedRows();
      const central: GridEditable = Metadata.getInstance('CENTRAL');
      
      editedRows.forEach((row: IDictionary) => {
        row.CDFILCENTPME = central.datasource.currentRow.CDFILIAL;
      });
      
      Http.post('/salvaAgregada', { editedRows }).then((response: IDictionary) => {
        if (response.data.length === 0) {
          grid.datasource.get();
          
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          this.gridEditableController.cancelEditing({ component });
        } else {
          const mensagem = response.data.dataset.mensagem[0];
          param = mensagem.split(':');
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_UNID_CENTRAL', { PARAM1: param[1] }), 'error');
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      });
    }
  }
  
  public confirmDeleteCentral({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiCentral',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteCentral(checkedRows, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteCentral(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/excluiCentral', { checkedRows }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        grid.datasource.get();
        grid.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
        DialogService.hide();
      } else {
        const mensagem = response.data.dataset.mensagem[0];
        this.utilities.showAlert('alertError', `pme::${mensagem}`, 'error');
        DialogService.hide();
      }
    }).catch((error) => {
      this.utilities.showAlert('alertError', error, 'error');
    });
  }
  
  public confirmDeleteAgregada({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiAgregada',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteAgregada(checkedRows, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteAgregada(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/excluiAgregada', { checkedRows }).then(() => {
      grid.datasource.get();
      grid.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
      DialogService.hide();
    }).catch((error) => {
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    });
  }

  public carregaAgregada({ component }: IEventParam<Datasource>) {
    const centralSelecionada = !!component.currentRow.CDFILIAL;

    if (centralSelecionada) {
      const agregada: Grid = Metadata.getInstance('AGREGADA');
    
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: component.currentRow.CDFILIAL,
        },
      ];
  
      agregada.datasource.filter = conditions;
      agregada.datasource.get();
    }
  }
}
