import {
  IEventParam,
  Loader,
  Metadata,
  IDictionary,
} from '@zeedhi/core';

import {
  GridEditable, Select, Button, Form,
} from '@zeedhi/common';
  
export class Ger80500NutricionistaController {
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  public crudController = Loader.getInstance('CrudController');
  
  public newRow = false;
  
  private utilities = Loader.getInstance('Utilities');
  
  public addRow({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formFilter');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');

    if (form.validate()) {
      if (!this.newRow) {
        if (unidade.value !== null) {
          this.gridEditableController.addRow({ component });
        }
      } else unidade.validations.required.message = 'pme::campoObrigatorio';
    }
  }
  
  public save({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formNutricionista');
    if (form.validate()) {
      const grid: GridEditable = Metadata.getInstance('gridNutricionista');
      const editedRows = grid.getEditedRows();
      const unidade = Metadata.getInstance<Select>('CDFILIAL');
      
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDNUTRICIO = row.CDOPERADOR.CDOPERADOR;
          row.CDFILIAL = unidade.value;
          row.NRORG = localStorage.getItem('NRORG');
        }
      });
      this.crudController.saveRows(editedRows, 'nutricio').then(() => {
        this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
        this.gridEditableController.defaultState(grid);
        this.gridEditableController.reload({ component });
      }).catch(() => {
        this.utilities.showAlert('Alert', 'pme::impossivelAdicionar', 'error');
      });
    }
  }

  public setFilterUnidade() {
    const addButton = Metadata.getInstance<Button>('addButton');
    const grid: GridEditable = Metadata.getInstance('gridNutricionista');
    const nrorg = localStorage.getItem('NRORG');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
    ];

    addButton.disabled = true;
    grid.datasource.filter = filter;
    grid.datasource.get().then(() => {
      if (unidade.value !== null) addButton.disabled = false;
    });
  }

  public setFilterOperador({ component }: IEventParam<Select>) {
    const nrorg = localStorage.getItem('NRORG');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
}
