import {
  Http,
  IDictionary,
  IEventParam,
  Loader,
  Metadata,
} from '@zeedhi/core';
import {
  Button,
  CollapseCard,
  DialogService,
  Form,
  GridEditable,
  IButtonEvent,
  INumber,
  Select,
  Tabs,
  Time,
} from '@zeedhi/common';

export class Pla01300PratoController {
  public crudController = Loader.getInstance('CrudController');
  
  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  public newRow = false;
  
  public editable = false;
  
  public central = localStorage.getItem('CDFILIAL');
  
  // Filtro Unidade
  public setFilterBranch({ component }: IEventParam<Select>) {
    const cdoperator = localStorage.getItem('CDOPERADOR');
    const nrorg = localStorage.getItem('NRORG');
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: cdoperator,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  // Adicionar na árvore de Prato
  // public addNewPlate() {
  //   const plateTree: GridEditable = Metadata.getInstance('PLATE_TREE');
  // this.geraCodigoPrato(plateTree);
  // VER SE VAI PRECISAR TER ESSE CONTROLE DE PERFIL
  // const row = {
  // COLUMN_NAME: 'CDGROPERREPA',
  // ALIAS_NAME: 'ALTRECEPADRAO',
  // };
  // Http.post('/retornaParametrosAdicionarPrato', 'Row', row).then((callback) => {
  //   if (callback && callback.dataset && callback.dataset.permission && callback.dataset.paramgeral.length) {
  // }
  // });
  // }
  
  // public geraCodigoPrato({ plateTree }: IEventParam<GridEditable>) {
  //   this.buscaNovoCodigo();
  //   plateTree.datasource.get();
  // }
  
  // public buscaNovoCodigo({ pratoPai, filAux, nivel, filial }: any) {
  //   const row = {
  //     PRATOPAI: pratoPai,
  //     CDFILAUXPRAT: filAux,
  //     NIVEL: nivel,
  //     FILIAL: filial,
  //   };
    
  //   return Http.post('buscaCodigoPrato', 'Row', row).then((callback) => { callback.dataset.CODIGO[0]; });
  // }
  
  // Seta Filtro do campo de unidade da Receita
  public setFilterBranchRecipe({ component }: IEventParam<Select>) {
    const cdoperator = localStorage.getItem('CDOPERADOR');
    const nrorg = localStorage.getItem('NRORG');
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: cdoperator,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  // Seta Filtro do campo serviço da Receita
  public setFilterServiceField({ component }: IEventParam<Select>) {
    const branch = component.value;
    const fieldService: Select = Metadata.getInstance('SERVICO');
    if (branch) {
      const nrorg = localStorage.getItem('NRORG');
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: branch.CDFILIAL,
        },
        {
          name: 'P_NRORG',
          operator: '=',
          value: nrorg,
        },
      ];
      fieldService.datasource.filter = conditions;
      fieldService.datasource.get();
    } else {
      fieldService.setValue('');
    }
  }
  
  // Seta Filtro da árvore de Prato
  public setFilterPlateTree({ component }: IEventParam<Select>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const cdfilial = component.value.CDFILIAL;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
    
    plateTree.datasource.filter = conditions;
    plateTree.datasource.get().then(() => {
      this.enabledButton(component);
      this.verifyVisibitityFieldsStandard();
    });
  }
  
  get defaultValueBranch() {
    return Metadata.getInstance<Select>('UNIDADE').value;
  }
  
  // Seta filtro do grid de Produtos da Receita Padrão
  public setfilterRecipeStandard({ component }: IEventParam<GridEditable>) {
    const gridRecipeStandard: GridEditable = Metadata.getInstance('RECIPE_STANDARD');
    const { currentRow } = component.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    gridRecipeStandard.datasource.filter = conditions;
    gridRecipeStandard.datasource.get().then(() => {
      this.loadStandardRecipeData(conditions);
      const addButton: Button = gridRecipeStandard.getChildInstance('addButton');
      addButton.disabled = false;
      this.enabledTabs();
    });
  }
  
  // Habilitar abas de Receita apenas
  public enabledTabs() {
    const tabsRecipes = Metadata.getInstance<Tabs>('tabsRecipes');
    const arrayTabs: IDictionary = tabsRecipes.tabs;
    arrayTabs.forEach((tab: IDictionary) => {
      tab.disabled = false;
    });
  }
  
  // Seta filtro do grid de Produtos da Receita Local
  public setFilterProductsRecipeLocal({ component }: IEventParam<Select>) {
    const gridRecipeLocal: GridEditable = Metadata.getInstance('RECIPE_LOCAL');
    const treeGridPlate: GridEditable = Metadata.getInstance('PLATE_TREE');
    const currentRow = treeGridPlate.datasource.currentRow;
    const cdfilial = component.value;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    gridRecipeLocal.datasource.filter = conditions;
    gridRecipeLocal.datasource.get().then(() => {
      this.loadLocalRecipeData(conditions);
      const addButton: Button = gridRecipeLocal.getChildInstance('addButton');
      addButton.disabled = false;
    });
  }

  // Seta filtro do grid de Produtos da Receita Serviço
  public setFilterProductsRecipeService({ component }: IEventParam<Select>) {
    const gridRecipeService: GridEditable = Metadata.getInstance('RECIPE_SERVICE');
    const treeGridPlate: GridEditable = Metadata.getInstance('PLATE_TREE');
    const currentRow = treeGridPlate.datasource.currentRow;
    const branch: Select = Metadata.getInstance('BRANCH_SERVICE');
    const cdservfili = component.value.CDSERVFILI;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: branch.value.CDFILIAL,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    gridRecipeService.datasource.filter = conditions;
    gridRecipeService.datasource.get().then(() => {
      this.loadServiceRecipeData(conditions);
      const addButton: Button = gridRecipeService.getChildInstance('addButton');
      addButton.disabled = false;
    });
  }
  
  // carrega os dados da Receita Padrão
  public loadStandardRecipeData(conditions: IDictionary) {
    const cardRecipeStandard = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_STANDARD');
    const formDadosRecipeStandard: Form = Metadata.getInstance('DATA_RECIPE_STANDARD');
    this.setFilterProductProduction(formDadosRecipeStandard, conditions);
    cardRecipeStandard.expanded = true;
    this.returnDataRecipe(conditions, 'find/dadosReceitaPadrao').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDadosRecipeStandard).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDadosRecipeStandard.value) {
            formDadosRecipeStandard.value[item] = response[item];
          }
        });
      }
    });
  }
  
  // Seta filtro de campo Produto Produção
  private setFilterProductProduction(form: Form, conditions: IDictionary) {
    const fieldProduct = form.getChildInstance<Select>('CDARVPROD');
    fieldProduct.datasource.filter = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: conditions[0].value,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: conditions[1].value,
      },
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: '',
      },
      {
        name: 'TCDPRODUTO',
        operator: '=',
        value: 'T',
      },
    ];
    fieldProduct.datasource.get();
  }
  
  private returnDataRecipe(conditions: IDictionary, route: string) {
    return Http.post(route, { requestType: 'FilterData', conditions }).then((response) => {
      const data = response.data.data[0];
      return data;
    });
  }
  
  public returnFieldsSelect(formDadosRecipeStandard: Form) {
    return formDadosRecipeStandard.children.filter((data) => data.component === 'ZdSelect');
  }
  
  // Carrega os dados da Receita Local
  public loadLocalRecipeData(conditions: IDictionary) {
    const cardRecipeLocal = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_LOCAL');
    const formDadosRecipeLocal: Form = Metadata.getInstance('DATA_RECIPE_LOCAL');
    this.setFilterProductProduction(formDadosRecipeLocal, conditions);
    cardRecipeLocal.expanded = true;
    this.returnDataRecipe(conditions, 'find/pratFiliProdAcabado').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDadosRecipeLocal).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDadosRecipeLocal.value) {
            formDadosRecipeLocal.value[item] = response[item];
          }
        });
      }
    });
  }
  
  // Carrega os dados da Receita Serviço
  public loadServiceRecipeData(conditions: IDictionary) {
    const cardRecipeService = Metadata.getInstance<CollapseCard>('CARD_DATA_RECIPE_SERVICE');
    const formDataRecipeService: Form = Metadata.getInstance('DATA_RECIPE_SERVICE');
    this.setFilterProductProduction(formDataRecipeService, conditions);
    cardRecipeService.expanded = true;
    this.returnDataRecipe(conditions, 'find/pratServProdAcabado').then((response) => {
      if (response) {
        this.returnFieldsSelect(formDataRecipeService).forEach((data) => {
          const nameResponse = data.name;
          if (response[nameResponse]) {
            Metadata.getInstance<Select>(nameResponse).setValue(response[nameResponse]);
            delete response[nameResponse];
          }
        });
        Object.keys(response).forEach((item) => {
          if (item in formDataRecipeService.value) {
            formDataRecipeService.value[item] = response[item];
          }
        });
      }
    });
  }
  
  // Verifica quais campos aparecerão no grid de receita padrão
  public verifyVisibitityFieldsStandard() {
    Http.post('/visibilityFields', { requestType: 'Row' }).then((response) => {
      const recipeStandard = Metadata.getInstance<GridEditable>('RECIPE_STANDARD');
      const resp: IDictionary = response.data.dataset.result;
      if (resp.IDQTLIQUBRUT === 'L') {
        recipeStandard.getChildInstance<INumber>('QTBRUTAPRUNR').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('QTBRUTAPRUNR').isVisible = false;
      }
      if (resp.IDAPRERECE === 'S') {
        recipeStandard.getChildInstance<Select>('NMAPRERECE').isVisible = true;
      } else {
        recipeStandard.getChildInstance<Select>('NMAPRERECE').isVisible = false;
      }
      if (resp.IDGANHOCOCC === 'S') {
        recipeStandard.getChildInstance<INumber>('VRPEFATCOCPR').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('VRPEFATCOCPR').isVisible = false;
      }
      if (resp.IDAPROVCOCC === 'S') {
        recipeStandard.getChildInstance<INumber>('VRPEFATCOCPRAP').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('VRPEFATCOCPRAP').isVisible = false;
      }
      if (resp.IDPERDALIMP === 'S') {
        recipeStandard.getChildInstance<INumber>('VRPEFATCORPR').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('VRPEFATCORPR').isVisible = false;
      }
      if (resp.IDAPROVLIMP === 'S') {
        recipeStandard.getChildInstance<INumber>('VRPEFATCORPRAP').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('VRPEFATCORPRAP').isVisible = false;
      }
      if (resp.IDTEMPPREP === 'S') {
        recipeStandard.getChildInstance<Time>('HRTEMPPREPPR').isVisible = true;
      } else {
        recipeStandard.getChildInstance<Time>('HRTEMPPREPPR').isVisible = false;
      }
      if (resp.IDUTREQCOMPENTD === 'S') {
        recipeStandard.getChildInstance<Select>('IDUTCOMPENTDIRP').isVisible = true;
      } else {
        recipeStandard.getChildInstance<Select>('IDUTCOMPENTDIRP').isVisible = false;
      }
      if (resp.IDUTDIASLIMSEG === 'S') {
        recipeStandard.getChildInstance<INumber>('QTLIMSEGPADR').isVisible = true;
      } else {
        recipeStandard.getChildInstance<INumber>('QTLIMSEGPADR').isVisible = false;
      }
    });
  }
  
  // Verifica quais campos aparecerão no grid de receita local
  public verifyVisibitityFieldsLocal() {
    Http.post('/visibilityFields', { requestType: 'Row' }).then((response) => {
      const recipeLocal = Metadata.getInstance<GridEditable>('RECIPE_LOCAL');
      const resp: IDictionary = response.data.dataset.result;
      if (resp.IDQTLIQUBRUT === 'L') {
        recipeLocal.getChildInstance<INumber>('QTBRUTALOUNR').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('QTBRUTALOUNR').isVisible = false;
      }
      if (resp.IDAPRERECE === 'S') {
        recipeLocal.getChildInstance<Select>('NMAPRERECE').isVisible = true;
      } else {
        recipeLocal.getChildInstance<Select>('NMAPRERECE').isVisible = false;
      }
      if (resp.IDGANHOCOCC === 'S') {
        recipeLocal.getChildInstance<INumber>('VRPEFATCOCLO').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('VRPEFATCOCLO').isVisible = false;
      }
      if (resp.IDAPROVCOCC === 'S') {
        recipeLocal.getChildInstance<INumber>('VRPEFATCOCLOAP').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('VRPEFATCOCLOAP').isVisible = false;
      }
      if (resp.IDPERDALIMP === 'S') {
        recipeLocal.getChildInstance<INumber>('VRPEFATCORLO').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('VRPEFATCORLO').isVisible = false;
      }
      if (resp.IDAPROVLIMP === 'S') {
        recipeLocal.getChildInstance<INumber>('VRPEFATCORLOAP').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('VRPEFATCORLOAP').isVisible = false;
      }
      if (resp.IDTEMPPREP === 'S') {
        recipeLocal.getChildInstance<Time>('HRTEMPPREPLO').isVisible = true;
      } else {
        recipeLocal.getChildInstance<Time>('HRTEMPPREPLO').isVisible = false;
      }
      if (resp.IDUTREQCOMPENTD === 'S') {
        recipeLocal.getChildInstance<Select>('IDUTCOMPENTDIRF').isVisible = true;
      } else {
        recipeLocal.getChildInstance<Select>('IDUTCOMPENTDIRF').isVisible = false;
      }
      if (resp.IDUTDIASLIMSEG === 'S') {
        recipeLocal.getChildInstance<INumber>('QTLIMSEGFILI').isVisible = true;
      } else {
        recipeLocal.getChildInstance<INumber>('QTLIMSEGFILI').isVisible = false;
      }
    });
  }
  
  // Verifica quais campos aparecerão no grid de receita serviço
  public verifyVisibitityFieldsService() {
    Http.post('/visibilityFields', { requestType: 'Row' }).then((response) => {
      const recipeService: GridEditable = Metadata.getInstance('RECIPE_SERVICE');
      const resp: IDictionary = response.data.dataset.result;
      if (resp.IDQTLIQUBRUT === 'L') {
        recipeService.getChildInstance<INumber>('QTBRUTASEUNR').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('QTBRUTASEUNR').isVisible = false;
      }
      if (resp.IDAPRERECE === 'S') {
        recipeService.getChildInstance<Select>('NMAPRERECE').isVisible = true;
      } else {
        recipeService.getChildInstance<Select>('NMAPRERECE').isVisible = false;
      }
      if (resp.IDGANHOCOCC === 'S') {
        recipeService.getChildInstance<INumber>('VRPEFATCOCSE').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('VRPEFATCOCSE').isVisible = false;
      }
      if (resp.IDAPROVCOCC === 'S') {
        recipeService.getChildInstance<INumber>('VRPEFATCOCSEAP').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('VRPEFATCOCSEAP').isVisible = false;
      }
      if (resp.IDPERDALIMP === 'S') {
        recipeService.getChildInstance<INumber>('VRPEFATCORSE').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('VRPEFATCORSE').isVisible = false;
      }
      if (resp.IDAPROVLIMP === 'S') {
        recipeService.getChildInstance<INumber>('VRPEFATCORSEAP').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('VRPEFATCORSEAP').isVisible = false;
      }
      if (resp.IDTEMPPREP === 'S') {
        recipeService.getChildInstance<Time>('HRTEMPPREPSE').isVisible = true;
      } else {
        recipeService.getChildInstance<Time>('HRTEMPPREPSE').isVisible = false;
      }
      if (resp.IDUTREQCOMPENTD === 'S') {
        recipeService.getChildInstance<Select>('IDUTCOMPENTDIRS').isVisible = true;
      } else {
        recipeService.getChildInstance<Select>('IDUTCOMPENTDIRS').isVisible = false;
      }
      if (resp.IDUTDIASLIMSEG === 'S') {
        recipeService.getChildInstance<INumber>('QTLIMSEGSERV').isVisible = true;
      } else {
        recipeService.getChildInstance<INumber>('QTLIMSEGSERV').isVisible = false;
      }
    });
  }
  
  public enabledButton(component: Select) {
    const addButton = Metadata.getInstance<Button>('addButtonTree');
    if (component.value) {
      addButton.disabled = false;
    } else {
      addButton.disabled = true;
    }
  }
  
  public deleteCheckedRows({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length) {
      Loader.getInstance('GridEditableController').deleteCheckedRows(component);
    } else {
      this.utilities.showAlert('Alert', 'pme::selecioneRegistro', 'error');
    }
  }
  
  // Carrega Unidade de Medida no grid de Receita
  // public carregaUnidadeConversao({ component }: IEventParam<Select>) {
  // EVENTO DE CLICK NO SELECT PENDENTE;
  // const row = component.parent.editedRows();
  // const nmColumProd = 'NMPRODUTO_editable_undefined';
  
  // Http.post('retornaFatorConversaoProduto', { requestType: 'Row', row }).then((callback) => {
  //   const dataSet = callback.data.dataset.INFOUNIDADE;
  //   component.datasource.data = dataSet;
  //   component.datasource.get();
  // });
  // }

  // Seta fields de unidade e dataSourceFilter de apresentação de acordo com Produto escolhido;
  public setFilterDependentsFields({ component }: IEventParam<Select>) {
    const produto = component.value;
    const nmColumnApres = 'NMAPRERECE_editable_undefined';
    const nmColumnUnidProd = 'SGUNIDADEPROD_editable_undefined';
    const apresentacao = component.parent?.getChildInstance<Select>(nmColumnApres);
    const sgUnidadeProd = Metadata.getInstance<Select>(nmColumnUnidProd);
    if (apresentacao) {
      apresentacao.datasource.filter = [
        {
          name: 'P_CDPRODUTO',
          operator: '=',
          value: produto.CDPRODUTO,
        },
      ];
      apresentacao.datasource.get().then(() => {
        apresentacao.disabled = false;
      });
      sgUnidadeProd.setValue(produto.SGUNIDADEPROD);
    }
  }

  // Seta fields de quantidades quesão dependentes de quantidade Bruta;
  public setQtdsDependents({ component }: IEventParam<INumber>) {
    const qtdeBruta = component.value;
    const columns = [
      'QTPRODPRAT_editable_undefined',
      'QTBRUTAPR_editable_undefined',
      'QTCALCNUTRPR_editable_undefined',
    ];
    columns.forEach((column) => {
      const field = component.parent?.getChildInstance<INumber>(column);
      if (field) field.value = qtdeBruta;
    });
  }
  
  // Cancela edição de Dados da Receita Padrão
  public cancelEditDataRecipeStandard({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadStandardRecipeData(conditions);
    component.disabled = true;
  }
  
  // Cancela edição de Dados da Receita Local
  public cancelEditDataRecipeLocal({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadLocalRecipeData(conditions);
    component.disabled = true;
  }
  
  // Cancela edição de Dados da Receita Serviço
  public cancelEditDataRecipeService({ component }: IEventParam<Button>) {
    const plateTree = Metadata.getInstance<GridEditable>('PLATE_TREE');
    const { currentRow } = plateTree.datasource;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT ? currentRow.CDFILAUXPRAT : 'P',
      },
    ];
    this.loadServiceRecipeData(conditions);
    component.disabled = true;
  }
  
  // Salva Dados da Receita Padrao
  public saveDataRecipeStandard() {
    const formDadosRecipeStandard = Metadata.getInstance<Form>('DATA_RECIPE_STANDARD');
    const row = formDadosRecipeStandard.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeStandard', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      }
    });
  }
  
  // Salva Dados da Receita Local
  public saveDataRecipeLocal() {
    const formDadosRecipeLocal = Metadata.getInstance<Form>('DATA_RECIPE_LOCAL');
    const row = formDadosRecipeLocal.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeLocal', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      }
    });
  }
  
  // Salva Dados da Receita Servico
  public saveDataRecipeService() {
    const formDadosRecipeService = Metadata.getInstance<Form>('DATA_RECIPE_SERVICE');
    const row = formDadosRecipeService.value;
    row.__is_new = false;
    Http.post('/saveDataRecipeService', { requestType: 'Row', row }).then((response) => {
      if (response.statusText === 'OK') {
        this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      }
    });
  }
  
  // Salvar grid de receita
  public saveRecipe({ component }: IButtonEvent) {
    const grid = component.parent as GridEditable;
    let form = Metadata.getInstance<Form>('formRecipeStandard');
    let tipoRec = '';
    
    switch (grid.name) {
      case 'RECIPE_STANDARD':
        tipoRec = 'P';
        break;
      case 'RECIPE_LOCAL':
        form = Metadata.getInstance<Form>('formRecipeLocal');
        tipoRec = 'L';
        break;
      case 'RECIPE_SERVICE':
        form = Metadata.getInstance<Form>('formRecipeService');
        tipoRec = 'S';
        break;
      default:
        break;
    }
    
    if (form.validate()) {
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.TIPORECEITA = tipoRec;
      });
      this.callSave('/saveRecipe', rows, grid);
    }
  }
  
  // Save grid
  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.editable = false;
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }
  
  // Seta Filtro do Produto de Estoque
  public setFilterProduct({ component }: IEventParam<Select>) {
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  // Confirma Exclusão de Produtos da Receita Padrão
  public confirmExclusionStandardRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosRecPadr',
        type: 'warning',
        title: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes', component: 'ZdButton', label: 'pme::sim', events: { click: () => this.deleteProductsRecipeStandard(gridRecipe, checkedRows) }, outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'pme::nao', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'pme::nenhumRegSelecionado', 'error');
    }
  }

  // Excluir produto da receita padrão
  public deleteProductsRecipeStandard(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const qtdProducts = gridRecipe.datasource.data.length;
    if ((qtdProducts - checkedRows.length) === 0) {
      this.utilities.showAlert('alert', 'pme::pratoSemReceitaPadrao', 'error');
      DialogService.hide();
    } else {
      const dataSourceName = '/recepadr';
      
      this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
        gridRecipe.cancelEditedRows();
        gridRecipe.datasource.get();
        gridRecipe.selectedRows = [];
        DialogService.hide();
        
        this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
        DialogService.hide();
      });
    }
  }
  
  // Confirma Exclusão de Produtos da Receita Local
  public confirmExclusionLocalRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosReclocal',
        type: 'warning',
        title: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes', component: 'ZdButton', label: 'pme::sim', events: { click: () => this.deleteProductsRecipeLocal(gridRecipe, checkedRows) }, outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'Não', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'pme::nenhumRegSelecionado', 'error');
    }
  }

  // Excluir produto da receita local
  public deleteProductsRecipeLocal(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const dataSourceName = '/recefili';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridRecipe.cancelEditedRows();
      gridRecipe.datasource.get();
      gridRecipe.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }

  // Confirma Exclusão de Produtos da Receita Serviço
  public confirmExclusionServiceRecipe({ component }: IEventParam<Button>) {
    const gridRecipe = component.parent as GridEditable;
    const checkedRows = gridRecipe.selectedRows;
    if (checkedRows.length > 0) {
      DialogService.show({
        name: 'excluiProdutosRecServ',
        type: 'warning',
        title: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes', component: 'ZdButton', label: 'pme::sim', events: { click: () => this.deleteProductsRecipeService(gridRecipe, checkedRows) }, outline: true,
          },
          {
            name: 'no', component: 'ZdButton', label: 'pme::nao', events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      this.utilities.showAlert('alertError', 'pme::nenhumRegSelecionado', 'error');
    }
  }

  // Excluir produto da receita serviço
  public deleteProductsRecipeService(gridRecipe: GridEditable, checkedRows: IDictionary) {
    const dataSourceName = '/receserv';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridRecipe.cancelEditedRows();
      gridRecipe.datasource.get();
      gridRecipe.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  // public calculateQtyPerPerson({ component }: IEventParam<Button>) {
  //   let gridRecipe = '';
  //   let prodRecipe = '';
  //   let qtAtiva = '';
  //   const sgunidade = '';
    
  //   switch (component.name) {
  //     case 'CALCULOPADR':
  //       gridRecipe = Metadata.getInstance<GridEditable>('RECIPE_STANDARD');
  //       prodRecipe = gridRecipe.datasource.data;
        
  //       if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTAPRUNR';
  //       else qtAtiva = 'QTPRODPRAT';
        
  //       break;
  //     case 'CALCULOLOC':
  //       gridRecipe = Metadata.getInstance<GridEditable>('RECIPE_LOCAL');
  //       prodRecipe = gridRecipe.datasource.data;

  //       if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTALOUNR';
  //       else qtAtiva = 'QTPRODRECLOC';

  //       break;
  //     case 'CALCULOSERV':
  //       gridRecipe = Metadata.getInstance<GridEditable>('RECIPE_SERVICE');
  //       prodRecipe = gridRecipe.datasource.data;

  //       if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTASEUNR';
  //       else qtAtiva = 'QTPRODRECSER';

  //       break;
  //     default:
  //       break;
  //   }
  //   this.retornaFatorConversaoPrato(prodRecipe, sgunidade, qtAtiva);
  // }
  
  public retornaFatorConversaoPrato(produtosRec: object, unPrat: string, qtAtiva: string) {
    const row = {
      PRODUTOS: produtosRec,
      SGUNIDADEPRAT: unPrat,
      QTATIVA: qtAtiva,
    };
    Http.post('/retornaFatorConversaoPrato', { requestType: 'Row', row }).then((callback) => callback.dataset.QTPESSOA[0]);
  }

  public addRow({ component }: IButtonEvent) {
    this.editable = true;
    const grid = component.parent as GridEditable;

    if (!this.gridEditController.newRow) {
      const item: IDictionary = {};
      if (grid.name === 'RECIPE_STANDARD') {
        item.IDQTDEFIXPR = 'N';
        item.IDTPREQPRODPADR = 'P';
        item.IDUTCOMPENTDIRP = 'N';
        item.IDPRODBASEPR = 'N';
        item.VRPEFATCOCPRAP = 100;
        item.VRPEFATCORPRAP = 100;
        item.VRPEFATNUTPR = 100;
      } else if (grid.name === 'RECIPE_LOCAL') {
        item.IDQTDEFIXLO = 'N';
        item.IDTPREQPRODFILI = 'P';
        item.IDUTCOMPENTDIRF = 'N';
        item.IDPRODBASELO = 'N';
        item.VRPEFATCOCLOAP = 100;
        item.VRPEFATCORLOAP = 100;
        item.VRPEFATNUTLO = 100;
      } else {
        item.IDQTDEFIXSE = 'N';
        item.IDTPREQPRODSERV = 'P';
        item.IDUTCOMPENTDIRS = 'N';
        item.IDPRODBASESE = 'N';
        item.VRPEFATCOCSEAP = 100;
        item.VRPEFATCORSEAP = 100;
        item.VRPEFATNUTSE = 100;
      }
      
      grid.datasource.data.unshift(item);
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }
}
