import {
  IEventParam,
  Loader,
  Metadata,
  IDictionary,
  Config,
} from '@zeedhi/core';

import {
  GridEditable, Select, Form, TextInput,
} from '@zeedhi/common';

import { PmeMask } from '@/config';
  
export class Ger30000PadraoDeServicoController {
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');
  
  public fillCode({ component }: IEventParam<TextInput>) {
    component.value = component.value.padStart(5, '0');
  }

  public formatCode() {
    const form = Metadata.getInstance<Form>('formPadrao');
    if (form.validate()) {
      const grid: GridEditable = Metadata.getInstance('grid');
      const editedRows: IDictionary = grid.getEditedRows();
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDSERVPADR = row.CDSERVPADR.padStart(5, '0');
        }
      });
      this.crudController.saveRows(editedRows, 'servpadr').then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public filterProd({ component }: IEventParam<Select>) {
    component.datasource.get();
  }
}
