import {
  Modal, ModalService, Form,
  Grid, IButtonEvent, IGridEvent,
  Text, TextInput, DialogService,
  moment, Button, Select,
} from '@zeedhi/common';
import {
  Config, Loader, Metadata, Http,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
import { PmeMask } from '@/config';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pla00115DadosDoPrato {
    private dadosPrato!: Modal;
    
    private selectedPlate!: IDictionary;
    
    private idErrorMessage!: string;
    
    private productError!: IDictionary;
    
    public showCost = false;
    
    private typePlate!: IDictionary;
    
    private utilities = Loader.getInstance('Utilities');
    
    private showGroup = false;
    
    private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
    
    public async createModal() {
      this.dadosPrato = await ModalService.createFromJson('modals/pla00115_dados_do_prato', true);
    }
    
    public destroyModal(onInit?: unknown) {
      this.dadosPrato.destroy();
      
      if (typeof onInit !== 'boolean') {
        const cardapio: Cardapio = Metadata.getInstance('cardapio');
        cardapio.setValueAction(false);
      }
    }
    
    public setFilterGroup({ component }: IEventParam<Select>) {
      component.datasource.filter = {
        CDFILAUXPRAT: (this.elaboracaoCardapio.getFilterData.aggregated !== null) ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL : this.elaboracaoCardapio.getFilterData.cdfilcentpme,
        CDNVPRATO: (Config.masks.plateMask as PmeMask).nivelMax,
      };
      component.datasource.get();
    }
    
    private setShowGroup() {
      this.showGroup = (this.elaboracaoCardapio.getFilterData.IDCONSGRPRAT !== 'N');
    }
    
    public async showModal(typePlate: IDictionary) {
      if (this.dadosPrato) this.destroyModal(true);
      await this.createModal();
      this.dadosPrato.show();
      this.setShowGroup();
      this.typePlate = typePlate;
      this.showCost = this.elaboracaoCardapio.getFilterData.searchPlate;
    }
    
    public fullscreen({ component }: IButtonEvent) {
      const modal: IDictionary = Metadata.getInstance('modalHeader');
      modal.parent.fullscreen = !modal.parent.fullscreen;
      component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
    }
    
    public filterDataPlate() {
      const gridPlate = Metadata.getInstance<Grid>('gridPlate');
      const route = (this.showCost) ? '/find/dadosDoPratoCusto' : '/find/dadosDoPrato';
      const filter = this.getFilterData();
      
      (gridPlate.datasource as IDictionary).route = route;
      gridPlate.datasource.filter = filter;
      gridPlate.datasource.get().then(() => {
        Metadata.getInstance<Text>('valueTypePlate').text = this.typePlate.NMTIPOPRAT;
        Metadata.getInstance<Text>('valueData').text = moment(this.typePlate.DTCARDIAPME, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
        this.selectFirstData(gridPlate);
      });
    }
    
    private getFilterData() {
      const filter: IDictionary = [];
      const selectGroup = Metadata.getInstance<Select>('selectGroup').value;
      if (this.showCost) {
        this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', selectGroup);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', this.typePlate.CDFILIAL);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
        this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI);
        this.utilities.onMountedFilterFind(filter, 'P_CDTIPOCARD', '=', this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILAUXPRAT', '=', (this.elaboracaoCardapio.getFilterData.aggregated !== null) ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL : this.elaboracaoCardapio.getFilterData.cdfilcentpme);
        this.utilities.onMountedFilterFind(filter, 'P_COMENSAIS', '=', this.typePlate.QTCOMESTCPME);
        this.utilities.onMountedFilterFind(filter, 'P_NIVEL', '=', (Config.masks.plateMask as PmeMask).nivelMax);
        this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.elaboracaoCardapio.getFilterData.planned);
        this.utilities.onMountedFilterFind(filter, 'P_DTCUSTOPROD', '=', (this.elaboracaoCardapio.getFilterData.radioSelect === 'R') ? this.elaboracaoCardapio.getFilterData.referenceDate : this.typePlate.DTCARDIAPME);
      } else {
        this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', selectGroup);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', this.typePlate.CDFILIAL);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
        this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI);
        this.utilities.onMountedFilterFind(filter, 'P_CDTIPOCARD', '=', this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILAUXPRAT', '=', (this.elaboracaoCardapio.getFilterData.aggregated !== null) ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL : this.elaboracaoCardapio.getFilterData.cdfilcentpme);
        this.utilities.onMountedFilterFind(filter, 'P_COMENSAIS', '=', this.typePlate.QTCOMESTCPME);
        this.utilities.onMountedFilterFind(filter, 'P_NIVEL', '=', (Config.masks.plateMask as PmeMask).nivelMax);
      }
      
      return filter;
    }
    
    private selectFirstData(gridPlate: Grid) {
      const selectRow = (this.typePlate.CDPRATO === null) ? undefined : gridPlate.datasource.data.filter((data: IDictionary) => data.CDPRATO === this.typePlate.CDPRATO)[0];
      gridPlate.datasource.currentRow = (selectRow !== undefined) ? selectRow : gridPlate.datasource.data[0];
      this.selectedPlate = gridPlate.datasource.currentRow;
      this.setValueDiners(gridPlate.datasource.currentRow.COMENSAL, gridPlate.datasource.currentRow.PERCAPLIC);
    }
    
    private setValueDiners(diners: string, percAplic: number) {
      const formModal = Metadata.getInstance<Form>('formPlate');
      formModal.value.numberDiners = this.getPerCons(parseInt(diners, 10));
      formModal.getChildInstance<TextInput>('numberDiners').label = (this.elaboracaoCardapio.getFilterData.IDPERCCOME === '%') ? 'pme::comensais%' : 'pme::comensais';
      if (percAplic !== null) formModal.getChildInstance<TextInput>('numberDiners').label += 'pme::percAplic';
    }
    
    private getPerCons(diners: number): number {
      let VRPECONSCPME = 0;
      if (this.elaboracaoCardapio.getFilterData.IDPERCCOME === '%') VRPECONSCPME = (this.typePlate.VRPECONSCPME === null) ? 100 : this.typePlate.VRPECONSCPME;
      else VRPECONSCPME = (this.typePlate.VRPECONSCPME === null && this.typePlate.QTCOMESTCPME === 0) ? this.typePlate.QTCOMESTCPME : ((this.typePlate.VRPECONSCPME / 100) * this.typePlate.QTCOMESTCPME);
      
      return (VRPECONSCPME === 0) ? diners : VRPECONSCPME;
    }
    
    public selectPlate({ row }: IGridEvent) {
      if (row.IDRECEITA !== null) {
        this.selectedPlate = row;
        this.mountFilter(row);
        Http.post('/recipeValidate', { requestType: 'Row', row }).then((response) => {
          const ativpr = response.data.dataset.recipe.filter((data: IDictionary) => data.IDPRODATIVPR === 'N');
          const ativ = response.data.dataset.recipe.filter((data: IDictionary) => data.IDPRODATIV === 'I');
          if (ativpr.length > 0) {
            this.selectedPlate = {};
            this.idErrorMessage = 'N';
            this.productError = ativpr;
          } else if (ativ.length > 0) {
            this.selectedPlate = {};
            this.idErrorMessage = 'I';
            this.productError = ativ;
          }
        });
      } else this.selectedPlate = {};
    }
    
    private mountFilter(row: IDictionary) {
      row.CDNVPRODUTO = (Config.masks.productMask as PmeMask).nivelMax;
      row.CDFILIAL = this.typePlate.CDFILIAL;
      row.CDSERVFILI = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
      row.CDFILCENTPME = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      row.CDTIPOCARD = this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD;
    }
    
    public saveModal({ component }: IEventParam<Button>) {
      const formModal = Metadata.getInstance<Form>('formPlate');
      if (Object.keys(this.selectedPlate).length > 0) { // Verifica se possui dados para inserção
        if (formModal.value.numberDiners > 0) { // Verifica se o numero de comensais foi inserido
          formModal.value.selectedPlate = this.selectedPlate; // Prato que foi selecionado para ser inserido
          formModal.value.plateData = formModal.value.checkboxDays.length > 0 ? this.getContentForDay(formModal.value) : [[this.typePlate]]; // Dados onde serão inseridos os pratos
          const row = formModal.value;
          this.calcPercent(row);
          this.utilities.activeLoadingButton(component);
          this.callSavePlateData(row).then((savePlateData) => {
            if (savePlateData) {
              this.updateRegisterCard(formModal.value);
              this.destroyModal();
              this.utilities.showAlert('plateSaved', 'pme::salvoComSucesso', 'success');
            }
          }).finally(() => {
            this.utilities.disableLoadingButton(component);
          });
        } else {
          this.utilities.showAlert('nullDiner', 'pme::comensalMenorIgualZero', 'error');
        }
      } else {
        this.errorSaveModal();
      }
    }
    
    public callSavePlateData(row: IDictionary) {
      return Http.post('plate/new', { requestType: 'Row', row }).then((response) => {
        const plateData = response.data.dataset.plateSaved[0];
        return plateData;
      });
    }
    
    private updateRegisterCard(formModal: IDictionary) {
      if (formModal.checkboxDays.length === 0) {
        formModal.plateData.forEach((typePlate: IDictionary) => {
          typePlate[0].CDPRATO = formModal.selectedPlate.CDPRATO;
          typePlate[0].NMPRATO = formModal.selectedPlate.NMPRATO;
          typePlate[0].CDFILAUXPRAT = formModal.selectedPlate.CDFILAUXPRAT;
        });
        
        const cardapio: Cardapio = Metadata.getInstance('cardapio');
        cardapio.forceUpdateInCard();
      } else {
        this.elaboracaoCardapio.reloadCard();
      }
    }
    
    private errorSaveModal() {
      if (this.idErrorMessage === 'N') {
        this.utilities.showAlert('inativo', 'pme::produtoInativoReceita', 'error');
        this.showDialog();
      } else if (this.idErrorMessage === 'I') {
        this.utilities.showAlert('inativo', 'pme::produtoInativoUnidade', 'error');
        this.showDialog();
      } else {
        this.utilities.showAlert('withoutRecipe', 'pme::semReceita', 'error');
      }
    }
    
    private calcPercent(row: IDictionary) {
      // Se for %, calcula a porcentagem inserida.
      let percent = 0;
      if (this.typePlate.QTCOMESTCPME !== 0) {
        percent = (this.elaboracaoCardapio.getFilterData.IDPERCCOME === 'C') ? parseFloat(((row.numberDiners * 100) / this.typePlate.QTCOMESTCPME).toFixed(2)) : row.numberDiners;
      }
      
      row.plateData.forEach((plate: IDictionary) => {
        plate[0].WEEKDAY = moment(plate[0].DTCARDIAPME, 'DD/MM/YYYY').day();
        plate[0].filterData = this.elaboracaoCardapio.getFilterData;
        plate[0].IDCARDESPEC = this.typePlate.IDCARDESPEC;
        plate[0].CDFILCENTPME = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
        plate[0].NRCARDPME = this.typePlate.NRCARDPME;
        plate[0].CDSERVFILI = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
        plate[0].VRPECONSCPME = (plate[0].DTCARDIAPME === this.typePlate.DTCARDIAPME) ? percent : 100;
      });
    }
    
    private getContentForDay(dataForm: IDictionary) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const datesInDay: IDictionary = [];
      const plateInDay: IDictionary = [];
      dataForm.checkboxDays.forEach((weekday: string) => {
        const daySelected = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME;
        const dataSet = cardapio.dates.filter((dates: IDictionary) => dates.WEEKDAY === parseInt(weekday, 10) && dates.DTCARDIAPME !== daySelected && dates.DTCARDIAPME > daySelected);
        dataSet.forEach((date: IDictionary) => datesInDay.push(date.DTCARDIAPME));
      });
      
      plateInDay.push([this.typePlate]);
      
      cardapio.menuContent.forEach((collumn: IDictionary) => {
        datesInDay.forEach((day: string) => {
          const plate = collumn.filter((plates: IDictionary) => plates.CDTIPOPRAT === this.typePlate.CDTIPOPRAT && plates.DTCARDIAPME === day && plates.NRSEQCNFCPME === this.typePlate.NRSEQCNFCPME);
          if (plate.length > 0) plateInDay.push(plate);
        });
      });
      
      return plateInDay;
    }
    
    private showDialog() {
      DialogService.show({
        name: 'pla00115_produtosInativos',
        type: 'error',
        title: 'pme::produtosInativos',
        text: this.textError(),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: 'pme::OK',
            events: { click: () => DialogService.hide() },
            outline: true,
          },
        ],
      });
    }
    
    // Monta a mensagem que será exibida no texto do dialogo quando existe produtos inativos.
    private textError() {
      let message = '';
      if (this.productError.length > 0) {
        this.productError.forEach((product: IDictionary) => {
          message += product.DESCRICAO;
          message += '<br>';
        });
      }
      return message;
    }
    
    public modifyColor({ row }: IGridEvent) {
      if (row.IDATIVO === 'N') return 'inactive-plate';
      return '';
    }
}
