import {
  Loader, Metadata, Http,
  IEventParam, IDictionary, I18n,
} from '@zeedhi/core';
import {
  CollapseCard, Text,
  Form, Select, DialogService,
  Progress, Date as ZdDate, Checkbox,
  Radio, Button, moment,
} from '@zeedhi/common';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00600ElaboracaoCardapio {
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');
  
  public showIcons = false;
  
  public filterData: IDictionary = {};
  
  public async createPeriodoModal() {
    this.modalController.createModal('/modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
  
  public filterCostType({ component }: IDictionary) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'STIDIOMA', '=', this.checkLanguage());
    
    component.datasource.filter = filter;
    component.datasource.get().then((data: IDictionary) => {
      if (data.length > 0) component.setValue(data[0][component.dataValue]);
    });
  }
  
  public openModalSearch() {
    const tipoCard: IDictionary = Metadata.getInstance('typeMenu');
    if (tipoCard.value) {
      this.modalController.openModal();
      this.utilities.setCdTipoCard(tipoCard.value.CDTIPOCARD);
    } else {
      this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    }
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    this.utilities.paramsCard = null;
  }
  
  private checkLanguage() {
    const language = localStorage.getItem('LANGUAGE') as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    
    return idioma;
  }
  
  public setFilterMenu({ component }: IDictionary) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral() ? this.appController.getCdCentral() : null);
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('typeMenu');
    const selectHeaderService = Metadata.getInstance<Select>('selectHeaderService');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral() ? this.appController.getCdCentral() : null);
    this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', cdtipoCard.value ? cdtipoCard.value.CDTIPOCARD : '');
    
    selectHeaderService.datasource.filter = filter;
    component.datasource.filter = filter;
    component.datasource.get();
    selectHeaderService.datasource.get();
  }
  
  public changeCardia({ component }: IEventParam<IDictionary>) {
    const menuForm = Metadata.getInstance<Form>('menuForm');
    if (this.utilities.paramsCard === null && component.value !== null) {
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
      this.utilities.onMountedFilterFind(filter, 'DTINICARDPME', '=', menuForm.value.initialDate);
      this.utilities.onMountedFilterFind(filter, 'DTFINCARDPME', '=', menuForm.value.finalDate);
      
      if (menuForm.value.initialDate !== null && menuForm.value.finalDate !== undefined) {
        if (menuForm.value.initialDate.length === 10 && menuForm.value.finalDate.length === 10) {
          if (this.utilities.dateIsValid(menuForm.value.initialDate, menuForm.value.finalDate)) {
            Http.post('/find/dadosCardapio', { requestType: 'FilterData', filter }).then((cardapio: IDictionary) => {
              if (cardapio.data.data.length > 0) {
                menuForm.value.NRCARDPME = cardapio.data.data[0].NRCARDPME;
                menuForm.value.describeCard = cardapio.data.data[0].DSCARDPME;
              } else {
                menuForm.value.NRCARDPME = null;
                menuForm.value.describeCard = null;
              }
            });
          }
        }
        
        if (menuForm.value.initialDate.length < 10) menuForm.value.finalDate = null;
      }
    }
  }
  
  public generateMenu({ component }: IEventParam<Button>) {
    const menuForm = Metadata.getInstance<Form>('menuForm');
    menuForm.value.cdfilcentpme = this.appController.getCdCentral();
    if (menuForm.validate()) {
      if (this.utilities.dateIsValid(menuForm.value.initialDate, menuForm.value.finalDate)) {
        this.utilities.getParamPlaneja(menuForm.value.aggregated?.CDFILIAL).then((paramplaneja: IDictionary) => { // Pega os dados da paramplaneja
          if (this.utilities.paramsCard !== null) menuForm.value.NRCARDPME = this.utilities.paramsCard.NRCARDPME;
          menuForm.value.IDCONSGRPRAT = paramplaneja.IDCONSGRPRAT; // Adicionar o paramentro IDCONSGRPRAT no objeto de filtro do cardapio...
          this.hasConfigCard({ component }, menuForm.value);
        });
      } else {
        this.utilities.showAlert('invalidDate', 'pme::dataInicialMaior', 'error');
      }
    }
  }
  
  // Verifica se o cardapio ja possui configuração
  private hasConfigCard({ component }: IEventParam<Button>, formValue: IDictionary) {
    const row = formValue;
    this.utilities.activeLoadingButton(component);
    Http.post('/hasConfigCard', { requestType: 'Row', row }).then((hasData) => {
      const perComenStorage = localStorage.getItem('IDPERCCOME');
      row.IDPERCCOME = (perComenStorage === null) ? hasData.data.dataset.hasConfigCard.IDPERCCOME : perComenStorage;
      
      if (formValue.NRCARDPME === null) { // Verifica se possui NRCARDPME
        this.callRulesMenu(formValue).then((response) => { if (response) this.newCard(formValue); }); // Verifica as regras para inserir um cardapio e insere
      } else if (hasData.data.dataset.hasConfigCard.hasData) { // Verifica se tem CONFCARDPME
        this.openMenu(formValue).then((resolve) => { // Abre o cardapio
          if (resolve.dishes.length > 0) {
            this.setValuesHeader(formValue, resolve.colors);
            const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
            if (collapse.expanded) this.toggle();
          }
        });
      } else {
        this.callRulesMenu(formValue).then((response) => { if (response) this.newCard(formValue); });
      }
    }).finally(() => {
      this.utilities.disableLoadingButton(component);
    });
  }
  
  private newCard(formValue: IDictionary) {
    const row = formValue;
    Http.post('/card/new', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.NRCARDPME.length > 0) {
        formValue.NRCARDPME = response.data.dataset.NRCARDPME[0];
        this.openMenu(formValue).then((resolve) => {
          if (resolve.dishes.length > 0) {
            this.setValuesHeader(formValue, resolve.colors);
            const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
            if (collapse.expanded) this.toggle();
          }
        });
      }
    });
  }
  
  private setValuesHeader(formValue: IDictionary, colors: IDictionary[]) {
    Metadata.getInstance<Text>('valueHeaderDescribe').text = formValue.describeCard;
    Metadata.getInstance<Select>('selectHeaderService').setValue(formValue.service);
    Metadata.getInstance<Text>('valueHeaderTypeMenu').text = formValue.typeMenu.DESCRICAO;
    Metadata.getInstance<Text>('valueHeaderPeriod').text = moment(formValue.initialDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    Metadata.getInstance<Text>('valueHeaderPeriod').text += ' ';
    Metadata.getInstance<Text>('valueHeaderPeriod').text += I18n.translate('pme::ate');
    Metadata.getInstance<Text>('valueHeaderPeriod').text += ' ';
    Metadata.getInstance<Text>('valueHeaderPeriod').text += moment(formValue.finalDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    
    Metadata.getInstance<Text>('headerReferenceDate').text = (formValue.radioSelect === 'R') ? 'pme::dataReferenciaCusto' : 'pme::dataCardapioCusto';
    Metadata.getInstance<Text>('valueHeaderReferenceDate').text = (formValue.radioSelect === 'R') ? moment(formValue.referenceDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : '';
    
    if (formValue.aggregated !== null) (document.getElementsByName('gridHeaderFilial')[0] as IDictionary).removeAttribute('style');
    else (document.getElementsByName('gridHeaderFilial')[0] as IDictionary).setAttribute('style', 'display: none');
    Metadata.getInstance<Text>('valueHeaderAggregated').text = (formValue.aggregated !== null) ? formValue.aggregated.CDNMFILIAL : '';
    
    if (colors.length) {
      const cor = colors[0].NRCORCARDESP;
      let hexadecimal = '#';
      hexadecimal += cor.toString(16);
      
      const elements = document.getElementsByClassName('zd-text corAgregada');
      Array.from(elements).forEach((data: IDictionary) => { data.style.color = hexadecimal; });
    } else {
      document.getElementsByName('headerAggregated')[0].style.color = '';
      document.getElementsByName('valueHeaderAggregated')[0].style.color = 'var(--zd-font-color)';
    }
  }
  
  // Pega todos os dados do menu parar poder abrir.
  public openMenu(formData: IDictionary) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    return this.getAllDataCard(formData).then((allData) => {
      this.orderContent(allData);
      this.getWeekDay(allData.dates);
      cardapio.cardData = allData.dishes;
      cardapio.dates = this.alterComens(allData.dates, allData.comens);
      cardapio.setMenuContent(this.joinEqualDate(allData.dates, allData.content, allData.dishes));
      cardapio.filterData = formData;
      cardapio.colorCard = allData.colors;
      
      // Verifica se possui cardapio para exibir o icones.
      if (formData.NRCARDPME !== null) this.showIcons = true;
      else this.showIcons = false;
      
      Metadata.getInstance<CollapseCard>('collapseCard').readonly = false; // Retira o readonly do filtro.
      if (Metadata.getInstance<Progress>('progressCard').isVisible) this.showReloadCard(); // Se tiver true, retira a barra de carregar
      
      this.filterData = formData;
      return allData;
    });
  }
  
  private getWeekDay(dataSet: IDictionary) {
    dataSet.forEach((data: IDictionary) => {
      data.WEEKDAY = moment(data.DTCARDIAPME, 'DD/MM/YYYY').day();
    });
  }
  
  private orderContent(allData: IDictionary) {
    const newArr: IDictionary = [];
    allData.dishes.forEach((dish: IDictionary) => {
      allData.content.forEach((menu: IDictionary) => {
        if (menu.NRORDCNFCPME === dish.NRORDCNFCPME) newArr.push(menu);
      });
    });
    allData.content = newArr;
  }
  
  private alterComens(date: IDictionary, comens: IDictionary) {
    date.forEach((data: IDictionary) => {
      comens.forEach((comensal: IDictionary) => {
        if (data.DTCARDIAPME === comensal.DTCARDIAPME) {
          data.QTCOMESTCPME = comensal.QTCOMESTCPME;
          data.IDCARDESPEC = comensal.IDCARDESPEC;
        }
      });
    });
    return date;
  }
  
  private joinEqualDate(dates: IDictionary, content: IDictionary, typeDish: IDictionary) {
    const newContent: IDictionary[] = [];
    const notFound = {
      VRPECONSCPME: 0,
    };
    
    dates.forEach((date: IDictionary) => {
      const auxContent = content.filter((response: IDictionary) => date.DTCARDIAPME === response.DTCARDIAPME);
      while (auxContent.length < typeDish.length) auxContent.push(notFound);
      newContent.push(auxContent);
    });
    
    return newContent;
  }
  
  private toggle() {
    const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
    collapse.expanded = !collapse.expanded;
    
    this.controllHeader(collapse.expanded);
  }
  
  public actionClickCollapse({ event }: IEventParam<CollapseCard>) {
    const iconClicked = (event as IDictionary).target.classList[0]; // Verifica se foi um icone que foi clicado
    if (iconClicked !== 'v-icon') this.controllHeader();
  }
  
  // Função para controlar os dados exibidos no header. Necessario saber se o header foi aberto pelo toogle ou por click
  public controllHeader(expanded?: boolean) {
    const collapseCard = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseCard.readonly) {
      const showCollapse = (typeof expanded !== 'boolean') ? !collapseCard.expanded : expanded;
      
      const formHeaderDisplay = document.getElementsByClassName('formHeaderDisplay');
      const collapseHeader: HTMLElement = document.getElementsByClassName('collapseHeaderDisplay')[0] as HTMLElement;
      
      if (showCollapse) {
        collapseHeader.style.display = this.getCssHeader(true);
        Array.from(formHeaderDisplay).forEach((html: IDictionary) => { html.style.display = this.getCssHeader(false); });
        this.calcMarginCardFilterOpen();
      } else {
        collapseHeader.style.display = this.getCssHeader(false);
        Array.from(formHeaderDisplay).forEach((html: IDictionary) => { html.style.display = this.getCssHeader(true); });
        this.calcMarginCardFilterClose();
      }
    }
  }
  
  // Ao diminuir o filtro, calcula o tamanho dele e adiciona como margin do component de cardapio e coloca o filtro como fixed.
  public calcMarginCardFilterClose() {
    const collapseCard = Metadata.getInstance<CollapseCard>('collapseCard');
    const cardapio = document.getElementsByClassName('card-container'); // Cardapio
    const divCalc = document.getElementsByClassName('v-expansion-panel-header'); // Header do filtro
    
    if (cardapio.length > 0) (cardapio[0] as IDictionary).style.margin = `${(divCalc[0] as IDictionary).offsetHeight + 13}px auto 0`; // Coloca a altura do header como margin do cardapio
    (document.getElementsByClassName('v-progress-linear')[0] as IDictionary).style.top = `${(divCalc[0] as IDictionary).offsetHeight + 13}px`;
    collapseCard.cssClass = 'fixedCollapse';
  }
  
  // Ao aumentar o filtro, calcula o tamanho dele e adiciona como margin do component de cardapio e coloca o filtro como relative.
  private calcMarginCardFilterOpen() {
    const collapseCard = Metadata.getInstance<CollapseCard>('collapseCard');
    const cardapio = document.getElementsByClassName('card-container');
    
    collapseCard.cssClass = 'relativeCollapse';
    (cardapio[0] as IDictionary).style.margin = '10px auto';
  }
  
  private getCssHeader(hasShow: boolean) {
    if (hasShow) return 'initial';
    return 'none';
  }
  
  /* Regras especificas do cardapio */
  /* Responsavel por verificar todas as regras do cardapio.
     (true) pode continuar a insercao
     (false) para de verificar para exibir a mensagem e nao pode inserir.
  */
  private callRulesMenu(form: IDictionary) {
    const row = form;
    return Http.post('/menuRules', { requestType: 'Row', row }).then((response) => {
      const message = response.data.dataset.message;
      if (!message.isCorrect) this.utilities.showAlert('callRuleError', message.message, 'error');
      
      return message.isCorrect;
    });
  }
  
  // Pega nos nomes dos tipos de pratos.
  private getAllDataCard(component: IDictionary) {
    const row = component;
    return Http.post('/card/get', { requestType: 'Row', row }).then((response) => {
      const allData = response.data.dataset.allData;
      allData.dishes.forEach((dish: IDictionary) => {
        if (dish.IDOPCCNFCPME === 'S') dish.NMTIPOPRAT += ' (Opção)';
      });
      return allData;
    });
  }
  
  public updateComen({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    const menuForm = Metadata.getInstance<Form>('menuForm');
    if (menuForm.value.NRCARDPME !== undefined) {
      this.utilities.getProcessDate().then((processDate: string) => {
        if (menuForm.value.aggregated !== null) {
          DialogService.show({
            name: 'pme00600_atualizarcomensais',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::desejaAtualizarComensais',
            buttons: [
              {
                name: 'yes',
                component: 'ZdButton',
                label: 'pme::sim',
                events: { click: () => this.updateComenAggregated(menuForm.value, processDate) },
                outline: true,
              },
              {
                name: 'no',
                component: 'ZdButton',
                label: 'pme::nao',
                events: { click: () => DialogService.hide() },
              },
            ],
          });
        } else {
          const modalAtualizaComensal = Loader.getInstance('Pme00600ModalAtualizaComensal');
          modalAtualizaComensal.showModal(menuForm.value, processDate);
        }
      });
    } else {
      this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
    }
  }
  
  private updateComenAggregated(valueForm: IDictionary, processDate: string) {
    const row = valueForm;
    row.DATAPROCE = processDate;
    Http.post('/procedureUpdateComen', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.updated[0]) this.utilities.showAlert('updated', 'pme::atualizadoComSucesso', 'success');
      DialogService.hide();
      this.reloadCard();
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // Atualiza o cardapio (Refaz a requisição dos dados filtrados)
  public reloadCard() {
    const menuForm = Metadata.getInstance<Form>('menuForm');
    this.openMenu(menuForm.value);
    this.showReloadCard();
  }
  
  // Essa função é chamada apenas quando é clicada com o mouse ou utilizada a keyMap.
  public reloadCardClickIcon({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.reloadCard();
  }
  
  public showReloadCard() {
    const reload = Metadata.getInstance<Progress>('progressCard');
    reload.isVisible = !reload.isVisible;
  }
  
  public disableDate({ component }: IEventParam<Checkbox>) {
    const radioDate = Metadata.getInstance<Radio>('radioSelect');
    
    if (component.value) {
      radioDate.disabled = false;
    } else {
      radioDate.disabled = true;
      Metadata.getInstance<ZdDate>('referenceDate').disabled = true;
    }
    
    this.changeStyleHeaderReference(component.value);
    this.firstDataRadio();
    if (radioDate.value === 'R' && component.value) this.ableReferenceDate();
    this.validateReferenceDate(radioDate.value);
  }
  
  private firstDataRadio() {
    const radioDate: IEventParam<Radio> = Metadata.getInstance('radioSelect');
    if (radioDate.value === undefined) radioDate.value = (radioDate.datasource as IDictionary).data[0].value;
  }
  
  public ableReferenceDate() {
    const radioDate = Metadata.getInstance<Radio>('radioSelect');
    const referenceDate = Metadata.getInstance<ZdDate>('referenceDate');
    
    if (radioDate.value === 'R') referenceDate.disabled = false;
    else referenceDate.disabled = true;
    
    this.validateReferenceDate(radioDate.value);
  }
  
  private validateReferenceDate(radioDate: string) {
    const referenceDate = Metadata.getInstance<ZdDate>('referenceDate');
    const searchPlate = Metadata.getInstance<Checkbox>('searchPlate');
    if (radioDate === 'R' && searchPlate.value) {
      (referenceDate as IDictionary).addValidation('required', { message: 'pme::campoObrigatorio' });
    } else {
      (referenceDate as IDictionary).rules = [];
      Metadata.getInstance<ZdDate>('referenceDate').disabled = true;
    }
  }
  
  // Se "exibir custos..." estiver selecionado irá aparecer no header
  private changeStyleHeaderReference(hasShow: boolean) {
    if (hasShow) (document.getElementsByName('gridHeaderReference')[0] as IDictionary).removeAttribute('style');
    else (document.getElementsByName('gridHeaderReference')[0] as IDictionary).setAttribute('style', 'display: none');
  }
  
  public changeStateBaseCost({ component }: IEventParam<Select>) {
    const baseCost = Metadata.getInstance<Checkbox>('baseCost');
    if (component.value) baseCost.disabled = false;
    else baseCost.disabled = true;
  }
  
  public filterAggregated({ component }: IEventParam<Select>) {
    component.datasource.filter = {
      CDFILCENTPME: this.appController.getCdCentral(),
    };
    component.datasource.get();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('service');
    const menuForm = Metadata.getInstance<Form>('menuForm');
    
    if (component.name === 'typeMenu') {
      servico.setValue('');
      menuForm.value.NRCARDPME = null;
      menuForm.value.describeCard = null;
      menuForm.value.finalDate = null;
      menuForm.value.initialDate = null;
    }
  }
  
  public get getFilterData() {
    return this.filterData;
  }
  
  public cancelEventHeader({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
  }
  
  public changeServiceFilter({ component }: IEventParam<Select>) {
    const service = Metadata.getInstance<Select>('service');
    if (component.value !== null) {
      if (service.value?.CDSERVFILI !== component.value?.CDSERVFILI) {
        service.setValue(component.value);
        (document.getElementsByName('selectHeaderService')[0] as HTMLElement).blur(); // Retira o foco do campo
        this.showReloadCard();
        const generateMenu = Metadata.getInstance<Button>('generateMenu');
        this.generateMenu({ component: generateMenu });
      }
    } else {
      service.setValue(component.value);
      (document.getElementsByName('selectHeaderService')[0] as HTMLElement).blur(); // Retira o foco do campo
      this.reloadCard();
    }
  }
}
