import {
  Modal, ModalService, Radio,
  CheckboxMultiple, SelectMultiple, Form,
  Button,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00600ExclusaoPrato {
    private modalExclusao!: Modal;
    
    public showSelectSpecific = false;
    
    public utilities = Loader.getInstance('Utilities');
    
    public async createModal() {
      this.modalExclusao = await ModalService.createFromJson('modals/pme00600_exclusao_prato', true);
    }
    
    public cancelModal() {
      if (this.modalExclusao) {
        this.modalExclusao.destroy();
        this.showSelectSpecific = false;
        const cardapio: Cardapio = Metadata.getInstance('cardapio');
        cardapio.setValueAction(false);
      }
    }
    
    public showModal({ event }: IEventParam<Button>) {
      (event as IDictionary).stopPropagation();
      this.validAndOpenModal();
    }
    
    public async validAndOpenModal() {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      if (cardapio.filterData.NRCARDPME !== undefined) {
        cardapio.setValueAction(true);
        await this.createModal();
        this.modalExclusao.show();
      } else {
        this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
      }
    }
    
    public changeVisibilitySelect({ component }: IEventParam<Radio>) {
      if (component.value === 'E') this.showSelectSpecific = true;
      else this.showSelectSpecific = false;
    }
    
    public showPlates({ component }: IEventParam<CheckboxMultiple>) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const newArray = JSON.parse(JSON.stringify(cardapio.cardData)); // Copia os dados retirando a referencia do objeto.
        
      this.modifyObjectCheckBox(newArray);
      (component.datasource as IDictionary).data = newArray;
      this.selectAllPlate(newArray);
    }
    
    // Marca todos os pratos do checkbox...
    private selectAllPlate(uniqueData: IDictionary) {
      const boxPlate = Metadata.getInstance<CheckboxMultiple>('checkboxPlates');
      uniqueData.forEach((data: IDictionary) => boxPlate.value.push(data.CDPRATSEQ));
    }
    
    // Faz a junção do tipo de prato e o sequencial, para a identificar quem é quem....
    private modifyObjectCheckBox(uniqueData: IDictionary) {
      uniqueData.forEach((data: IDictionary) => {
        data.CDPRATSEQ = `${data.CDTIPOPRAT} - ${data.NRSEQCNFCPME}`;
      });
    }
    
    public filterSelectMultiple({ component }: IEventParam<SelectMultiple>) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const filter = [
        {
          name: 'P_NRCARDPME',
          operator: '=',
          value: cardapio.filterData.NRCARDPME,
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: cardapio.filterData.cdfilcentpme,
        },
        {
          name: 'P_CDSERVFILI',
          operator: '=',
          value: cardapio.filterData.service.CDSERVFILI,
        },
      ];
      
      component.datasource.filter = filter;
      component.datasource.get();
    }
    
    public exclude({ component }: IEventParam<Button>) {
      const form: Form = Metadata.getInstance('formExclude');
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      
      if (form.validate()) {
        const initialDate = cardapio.dates.find((date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.initialDateExclude);
        const finalDate = cardapio.dates.find((date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.finalDateExclude);
        
        if (initialDate !== undefined && finalDate !== undefined) {
          if (!this.bloqDayInPeriod(form.value.initialDateExclude, form.value.finalDateExclude, cardapio)) {
            const row = form.value;
            row.platesSelect = Metadata.getInstance<CheckboxMultiple>('checkboxPlates').value;
            row.filterData = cardapio.filterData;
            this.utilities.activeLoadingButton(component);
            this.callExcludeRoute(row).then((excludeResponse: IDictionary) => {
              if (excludeResponse.type === 'success') {
                const elaboracao = Loader.getInstance('Pme00600ElaboracaoCardapio');
                elaboracao.reloadCard();
                this.cancelModal();
              }
              this.utilities.showAlert('excludePlate', excludeResponse.message, excludeResponse.type);
            }).finally(() => {
              this.utilities.disableLoadingButton(component);
            });
          } else {
            this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
          }
        } else {
          this.utilities.showAlert('outOfPeriod', 'pme::dataForaDoPeriodo', 'error');
        }
      }
    }
    
    private bloqDayInPeriod(initialDate: string, finalDate: string, cardapio: Cardapio): boolean {
      let hasBloqDay = false;
      cardapio.dates.forEach((date: IDictionary) => {
        if (date.DTCARDIAPME.substring(0, 10) >= initialDate && date.DTCARDIAPME.substring(0, 10) <= finalDate) {
          if (!cardapio.dateLessDateOfProcess(date.DTCARDIAPME.substring(0, 10))) hasBloqDay = true;
        }
      });
      return hasBloqDay;
    }
    
    public callExcludeRoute(row: IDictionary) {
      return Http.post('/excludePlates', { requestType: 'Row', row }).then((response) => {
        const excludeResponse = response.data.dataset.exclude;
        return excludeResponse;
      });
    }
    
    public defaultRadio({ component }: IEventParam<Radio>) {
      component.value = (component.datasource as IDictionary).data[0].value;
    }
}
