import {
  Datasource, Http, IDictionary, Loader, Metadata, IEventParam, Config,
} from '@zeedhi/core';
import {
  Button, CollapseCard, Date as ZdDate, Form, GridEditable, IButtonEvent, Select, TextInput, Modal, ModalService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme01400EfetivosController {
  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public editable = false;
  
  private dtProcessa = '';
  
  private modalConfigCard!: Modal;
  
  private nrlista = '';
  
  get controlExportButton() {
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    if (gridEfetivos.selectedRows.length === 0) return true;
    return false;
  }
  
  get controlAddButtonEfetivos() {
    const dtcardiapme = Metadata.getInstance<Select>('DTCARDIAPME');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const tipocard = Metadata.getInstance<Select>('TIPOCARD');
    if (dtcardiapme.value && unidade.value && tipocard.value) return false;
    return true;
  }
  
  public addRow({ component }: IButtonEvent) {
    this.editable = true;
    const grid = component.parent as GridEditable;

    if (!this.gridEditController.newRow) {
      const item = { __is_new: true };
      grid.datasource.data.unshift(item);
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }

  public cancelEditing({ component }: IButtonEvent) {
    this.editable = false;
    this.gridEditController.cancelEditing({ component });
  }
  
  public setDefaultValues({ component }: IEventParam<ZdDate>) {
    this.utilities.getProcessDate().then((processDate: string) => {
      const diaSemana = this.returnDiaSemana(processDate);
      this.dtProcessa = processDate;
      component.value = processDate;
      Metadata.getInstance<TextInput>('DIASEMANA').value = diaSemana;
    });
  }
  
  public validaData(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const dataComponente = row.DTCARDIAPME;
    const dataProcessamento = this.dtProcessa;
    const fieldDiaSemana = Metadata.getInstance<TextInput>('DIASEMANA');
    const dtTipoCard = Metadata.getInstance<Select>('DTCARDIAPME');
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    fieldDiaSemana.value = this.returnDiaSemana(dataComponente);
    if (!this.utilities.dateIsValid(dataComponente, dataProcessamento)) {
      this.utilities.showAlert('alertError', 'pme::dataMenorIgualDtProcessa', 'red');
      dtTipoCard.value = '';
      fieldDiaSemana.value = '';
      gridEfetivos.datasource.data = [];
    } else valid = true;
    return valid;
  }
  
  public setFilterTpCard({ component }: IEventParam<Select>) {
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'P_CDTIPOCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard.value,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: localStorage.getItem('NRORG'),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    const gridClientes = Metadata.getInstance<GridEditable>('CLIENTES_GRID');
    const unidade = Metadata.getInstance<Select>('UNIDADE');

    if (component.name === 'TIPOCARD') {
      unidade.setValue('');
      gridEfetivos.datasource.data = [];
      gridClientes.datasource.data = [];
    } else unidade.setValue('');
  }
  
  // Grid Efetivos
  
  public isDetailsActionVisible({ row }: IDictionary) {
    return row.IDSOMADIF === 'S';
  }
  
  public alertButtonAction() {
    this.utilities.showAlert('alertError', 'pme::qtdTotalEfetClie', 'warning');
  }
  
  public setFilterGridEfetivos() {
    const filtro = Metadata.getInstance<Form>('FILTRO');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const dtTipoCard = Metadata.getInstance<Select>('DTCARDIAPME');
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const row = filtro.value;
    if (this.validaData(filtro)) {
      if (unidade.value) {
        Http.post('/verificaExisteCardapioFiltro', { requestType: 'Row', row }).then((response) => {
          if (response.data.dataset.result.length !== 0) {
            const gridServEfet = Metadata.getInstance<Select>('EFETIVOS_GRID');
            const filter = [
              {
                name: 'CDFILCENTPME',
                operator: '=',
                value: this.appController.getCdCentral(),
              },
              {
                name: 'CDFILIAL',
                operator: '=',
                value: unidade.value.CDFILIAL ? unidade.value.CDFILIAL : '',
              },
              {
                name: 'DTCARDIAPME',
                operator: '=',
                value: dtTipoCard.value,
              },
              {
                name: 'DTEFETCLIE',
                operator: '=',
                value: dtTipoCard.value,
              },
              {
                name: 'CDTIPOCARD',
                operator: '=',
                value: tipoCard.value,
              },
            ];
            gridServEfet.datasource.filter = filter;
            gridServEfet.datasource.get().then(() => {
              const dados = gridServEfet.datasource.data;
              if (dados.length > 0) {
                gridServEfet.datasource.currentRow = dados[0];
                Metadata.getInstance<TextInput>('QTCLIESTCPME').value = gridServEfet.datasource.currentRow.QTCLIESTCPME;
                Metadata.getInstance<TextInput>('QTFUNESTCPME').value = gridServEfet.datasource.currentRow.QTFUNESTCPME;
                Metadata.getInstance<TextInput>('QTDOAESTCPME').value = gridServEfet.datasource.currentRow.QTDOAESTCPME;
                Metadata.getInstance<TextInput>('QTCOMESTCPME').value = gridServEfet.datasource.currentRow.QTCOMESTCPME;
              } else {
                Metadata.getInstance<TextInput>('QTCLIESTCPME').value = '';
                Metadata.getInstance<TextInput>('QTFUNESTCPME').value = '';
                Metadata.getInstance<TextInput>('QTDOAESTCPME').value = '';
                Metadata.getInstance<TextInput>('QTCOMESTCPME').value = '';
              }
            });
          } else {
            this.utilities.showAlert('alertError', 'pme::dataForaPeridoCard', 'red');
            tipoCard.setValue('');
            unidade.setValue('');
          }
        });
      } else {
        const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
        const gridClientes = Metadata.getInstance<GridEditable>('CLIENTES_GRID');
        gridEfetivos.datasource.data = [];
        gridClientes.datasource.data = [];
      }
    }
  }
  
  public saveServiceEfetivos({ component }: IEventParam<Button>) {
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    const dados = gridEfetivos.datasource.data;
    const filtro = Metadata.getInstance<Form>('FILTRO');
    const rows = gridEfetivos.getEditedRows();

    rows.forEach((item) => {
      item.CDFILCENTPME = this.appController.getCdCentral();
      item.CDFILIAL = filtro.value.UNIDADE.CDFILIAL;
      item.DTCARDIAPME = filtro.value.DTCARDIAPME;
      item.CDTIPOCARD = filtro.value.TIPOCARD;
      item.NRORG = localStorage.getItem('NRORG');
      item.NRCARDPME = dados.length > 1 ? dados[1].NRCARDPME : dados[0].NRCARDPME;
    });
    
    Http.post('/saveServEfetivos', { requestType: 'DataSet', rows }).then((response) => {
      const result = response.data.dataset.result;
      this.utilities.showAlert('alertError', result.message, result.type);
      this.gridEditController.defaultState(gridEfetivos);
      this.gridEditController.reload({ component });
    });
  }
  
  public saveClientes({ component }: IEventParam<Button>) {
    const gridClientes = Metadata.getInstance<GridEditable>('CLIENTES_GRID');
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    const dados = gridEfetivos.datasource.currentRow;
    const filtro = Metadata.getInstance<Form>('FILTRO');
    const formEstimados = Metadata.getInstance<Form>('formEfetivos');
    const rows = gridClientes.getEditedRows();
    
    rows.forEach((item) => {
      item.CDFILCENTPME = this.appController.getCdCentral();
      item.CDFILIAL = filtro.value.UNIDADE.CDFILIAL;
      item.DTEFETCLIE = filtro.value.DTCARDIAPME;
      item.DTCARDIAPME = filtro.value.DTCARDIAPME;
      item.CDTIPOCARD = filtro.value.TIPOCARD;
      item.NRORG = localStorage.getItem('NRORG');
      item.QTCLIESTCPME = formEstimados.value.QTCLIESTCPME;
      item.QTCOMESTCPME = formEstimados.value.QTCOMESTCPME;
      item.QTDOAESTCPME = formEstimados.value.QTDOAESTCPME;
      item.QTFUNESTCPME = formEstimados.value.QTFUNESTCPME;
      item.NRCARDPME = dados.NRCARDPME;
      item.NRCARDIAPME = dados.NRCARDIAPME;
      item.QTDOAEFECPME = dados.QTDOAEFECPME;
      item.QTCLIEFECPME = dados.QTCLIEFECPME;
      item.QTCOMEFECPME = dados.QTCOMEFECPME;
      item.QTFUNEFECPME = dados.QTFUNEFECPME;
    });
    
    Http.post('/saveClientes', { requestType: 'DataSet', rows }).then((response) => {
      const result = response.data.dataset.result;
      this.utilities.showAlert('alertError', result.message, result.type);
      this.gridEditController.defaultState(gridClientes);
      gridClientes.datasource.data = [];
      this.gridEditController.reload({ component });
      gridEfetivos.datasource.get();
    });
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const filtro = Metadata.getInstance<Form>('FILTRO');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: filtro.value.TIPOCARD,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: filtro.value.CDFILIAL,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterGridClients({ component }: IEventParam<Datasource>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const dtTipoCard = Metadata.getInstance<Select>('DTCARDIAPME');
    const filial = Metadata.getInstance<Select>('UNIDADE');
    const gridClientes = Metadata.getInstance<Select>('CLIENTES_GRID');
    this.setFormEstimados();
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: filial.value.CDFILIAL,
      },
      {
        name: 'DTEFETCLIE',
        operator: '=',
        value: dtTipoCard.value,
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard.value,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: component.currentRow.CDSERVFILI,
      },
    ];
    
    if (component.currentRow.CDSERVFILI) {
      gridClientes.datasource.filter = filter;
      gridClientes.datasource.get().then(() => {
        Metadata.getInstance<CollapseCard>('cliente-card').expanded = true;
      });
    } else {
      gridClientes.datasource.data = [];
    }
  }
  
  public returnDiaSemana(data: string) {
    const semana = ['Domingo', 'Segunda-Feira', 'Terça-Feira', 'Quarta-Feira', 'Quinta-Feira', 'Sexta-Feira', 'Sábado'];
    let result = '';
    if (data !== null) {
      const arr: IDictionary = data.split('/').reverse();
      const dateFormat = new Date(arr[0], arr[1] - 1, arr[2]);
      const dia = dateFormat.getDay();
      result = semana[dia];
    }

    return result;
  }
  
  public setFormEstimados() {
    const currentRow = Metadata.getInstance<GridEditable>('EFETIVOS_GRID').datasource.currentRow;
    Metadata.getInstance<TextInput>('QTCLIESTCPME').value = currentRow.QTCLIESTCPME;
    Metadata.getInstance<TextInput>('QTFUNESTCPME').value = currentRow.QTFUNESTCPME;
    Metadata.getInstance<TextInput>('QTDOAESTCPME').value = currentRow.QTDOAESTCPME;
    Metadata.getInstance<TextInput>('QTCOMESTCPME').value = currentRow.QTCOMESTCPME;
  }
  
  public enableExport() {
    Metadata.getInstance<Button>('exportButton').disabled = false;
  }
  
  // Exportar Efetivos
  public openExportModal() {
    this.modalController.openModal();
  }
  
  public async createModal() {
    this.modalController.createModal('/modals/formExportEfetivos');
    this.modalConfigCard = await ModalService.createFromJson('modals/closeButtonModalEfetivos', true);
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }
  
  public cancelModal() {
    const modal = Metadata.getInstance<Modal>('formExportEfetivos');
    const unidade = modal.getChildInstance<Select>('UNIDADE');
    this.modalConfigCard.hide();
    this.modalConfigCard.destroy();
    this.modalController.closeModal();
    unidade.setValue('');
  }
  
  public setFilterUnidadeModal() {
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    const modal = Metadata.getInstance<Modal>('formExportEfetivos');
    const row = gridEfetivos.selectedRows;
  
    Http.post('/buscaUnidade', { requestType: 'Row', row }).then((response) => {
      const unidadeModal = modal.getChildInstance<Select>('UNIDADE');
      const result = response.data.dataset.result.data;
      this.nrlista = response.data.dataset.result.nrlista;
      if (result.length > 0) {
        unidadeModal.datasource.updateData(result);
      } else {
        this.utilities.showAlert('alertError', 'pme::semUnidadeComServicoPrato', 'red');
      }
    });
  }
  
  public exportEfetivos({ component }: IEventParam<Button>) {
    const gridEfetivos = Metadata.getInstance<GridEditable>('EFETIVOS_GRID');
    const modal = Metadata.getInstance<Modal>('formExportEfetivos');
    const unidadeModal = modal.getChildInstance<Select>('UNIDADE');
    const row = gridEfetivos.selectedRows;
    
    row.forEach((item) => {
      item.unidade = unidadeModal.value;
      item.nrlista = this.nrlista;
    });
    
    this.utilities.activeLoadingButton(component);
    Http.post('/exportEfetivos', { requestType: 'Row', row }).then(() => {
      this.utilities.showAlert('alertError', 'pme::exportadoComSucesso', 'green');
      this.cancelModal();
      gridEfetivos.selectedRows = [];
    });
  }
  
  public clienteComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'NRINSJURCLIE',
        mask: (Config.masks.cnpjMask as PmeMask).pattern,
      },
      'NMRAZSOCCLIE',
    ],
    formatterDataText: [
      {
        name: 'NRINSJURCLIE',
        mask: (Config.masks.cnpjMask as PmeMask).pattern,
      },
      'NMRAZSOCCLIE',
    ],
    dataValue: 'CDCLIENTE',
    dense: true,
    returnObject: true,
    datasource: {
      uniqueKey: 'CDCLIENTE',
      route: '/clienteServicoEfetivos',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
