import {
  Datasource, Http, IDictionary, Loader, Metadata, IEventParam, Config,
} from '@zeedhi/core';
import {
  Button, Date as ZdDate, Form, GridEditable, Select, TextInput, CheckboxMultiple, DialogService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme02000RetiradaInventarioController {
  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');

  public dtIni = '';

  public dtFin = '';

  private situation = {
    PENDENTE: 'S',
    REALIZADO: 'N',
  };

  private currentRow: IDictionary = {};

  public disableFields = true;

  public showCost = 'N';
  
  public showItems = false;
  
  public itemSaved = false;

  public colsRomaneio = 12;

  public selectedItems: IDictionary = [];

  public prodEditable({ row }: IDictionary) {
    return row.__is_new;
  }

  public get visibilityItems() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    // if (gridRomaneio.datasource.currentRow.DTINVENTRETI) this.colsRomaneio = 12;
    // else this.colsRomaneio = 12;
    return !!gridRomaneio.datasource.currentRow.DTINVENTRETI;
  }
  
  public getConfigRetirada() {
    const central = this.appController.getCdCentral();
    if (central) {
      Http.post('/configRetirada', { CDCENTRAL: central }).then((response) => {
        const config = response.data.dataset.result;
        if (config[0] && config[0].IDPRINVENRETI === 'S') {
          this.disableFields = false;
          this.utilities.getProcessDate(central).then((date: string) => {
            this.dtIni = date;
            this.dtFin = date;
          });
        } else {
          this.disableFields = true;
          this.utilities.showAlert('erroConfig', 'pme::centralNaoHabilitadaRetirada', 'error');
        }
      });
    }
  }

  public setSituation({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;

      if (items.includes('PENDENTE')) this.situation.PENDENTE = 'S';
      else this.situation.PENDENTE = 'N';
      
      if (items.includes('REALIZADO')) this.situation.REALIZADO = 'S';
      else this.situation.REALIZADO = 'N';

      this.setFilterRomaneio();
    }
  }

  public setCost({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;
      this.showCost = items.includes('SHOWCUSTO') ? 'S' : 'N';
      this.reloadItems();
    }
  }

  public setFilterRomaneio() {
    const grid: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    
    let efetivados = null;
    if (this.situation.PENDENTE === 'S' && this.situation.REALIZADO === 'S') efetivados = 'T';
    else if (this.situation.PENDENTE === 'S') efetivados = 'N';
    else if (this.situation.REALIZADO === 'S') efetivados = 'S';

    const conditions = [
      {
        name: 'P_CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'P_DTINICIAL',
        operator: '=',
        value: this.dtIni,
      },
      {
        name: 'P_DTFINAL',
        operator: '=',
        value: this.dtFin,
      },
      {
        name: 'P_IDRETIEFET',
        operator: '=',
        value: efetivados,
      },
    ];

    grid.datasource.filter = conditions;
    grid.datasource.get();
  }

  public setFilterItems({ component }: IEventParam<Datasource>) {
    this.currentRow = component.currentRow;
    const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const toolbarSlot = grid.toolbarSlot;
    
    const disableAdd = this.currentRow.IDRETIEFET === 'S';
    toolbarSlot.forEach((e: IDictionary) => {
      if (e.name === 'addButton') e.disabled = disableAdd;
    });
    
    this.reloadItems();
  }

  public reloadItems() {
    const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const central = this.appController.getCdCentral();

    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: agregada.value,
        },
        {
          name: 'P_IDEXIBCUS',
          operator: '=',
          value: this.showCost,
        },
        {
          name: 'P_DTPOSIESTQ',
          operator: '=',
          value: this.currentRow.DTINVENTRETI,
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: central,
        },
        {
          name: 'P_NRINVENTRETI',
          operator: '=',
          value: this.currentRow.NRINVENTRETI,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get();
    }
  }

  public getFilterProdItem() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.currentRow.NRINVENTRETI ? this.currentRow.NRINVENTRETI : null,
      },
    ];
  }

  public setFilterProdForm({ component }: IEventParam<Select>) {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.romaneio.NRINVENTRETI ? this.romaneio.NRINVENTRETI : null,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public saveItems({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formItemsGrid = Metadata.getInstance<Form>('formItemsGrid');
    if (this.currentRow.IDRETIEFET === 'S') this.utilities.showAlert('errorEdicaoItem', 'pme::romaneioJaEfetivado', 'error');
    else if (formFiltro.validate() && formItemsGrid.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDFILIAL = formFiltro.value.UNIDADE;
        e.DTINVENTRETI = this.currentRow.DTINVENTRETI;
        e.NRINVENTRETI = this.currentRow.NRINVENTRETI;
      });
      this.callSave('/saveItemsRomaneio', rows, grid);
    }
  }

  public deleteItems({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    this.selectedItems = grid.selectedRows;
    if (this.currentRow.IDRETIEFET === 'N') {
      if (this.selectedItems.length === grid.datasource.total) {
        this.selectedItems = grid.selectedRows;
        DialogService.show({
          name: `${component.name}Lancamento`,
          type: 'warning',
          title: 'pme::atencao',
          text: 'pme::excluirUltimoItemLancamento',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'pme::sim',
              events: { click: () => this.callDeleteItems() },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'pme::nao',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      } else this.gridEditController.deleteCheckedRows({ component });
    } else this.utilities.showAlert('errorExclusaoItem', 'pme::romaneioJaEfetivado', 'error');
  }

  public callDeleteItems() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    this.selectedItems[0].CDFILIAL = agregada.value;

    Http.post('/deleteItemRomaneio', { requestType: 'Row', row: this.selectedItems }).then(() => {
      gridItem.cancelEditedRows();
      gridItem.datasource.get();
      gridItem.selectedRows = [];
      this.selectedItems = [];
      gridRomaneio.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public deleteRomaneio({ component }: IEventParam<Button>) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const checkedRows = gridRomaneio.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else if (!this.romaneioIsValid(checkedRows)) this.utilities.showAlert('alert', 'pme::romaneioJaEfetivado', 'warning');
    else {
      DialogService.show({
        name: `${component.name}`,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::excluirLancamento',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteRomaneio() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private romaneioIsValid(rows: IDictionary): boolean {
    let valid = true;
    rows.forEach((e: IDictionary) => {
      if (e.IDRETIEFET === 'S') valid = false;
    });
    return valid;
  }

  public callDeleteRomaneio() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const rows = gridRomaneio.selectedRows;

    Http.post('/deleteRomaneio', { requestType: 'Row', row: rows }).then(() => {
      gridRomaneio.cancelEditedRows();
      gridRomaneio.datasource.get();
      gridRomaneio.selectedRows = [];
      this.selectedItems = [];
      gridItem.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public itemIsValid({ row }: IDictionary) {
    let color = '';
    if (this.currentRow.IDRETIEFET === 'N' && row.QTESTOQDIA < row.QTITINVENTRETI) color = 'red';
    return color;
  }

  public cssEfetivado({ row }: IDictionary) {
    let colorClass = 'green-efetivos';
    if (row.IDRETIEFET === 'N') colorClass = 'red-efetivos';
    return colorClass;
  }

  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodUltimoNivelItem',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.utilities.showAlert('alertSuccess', 'pme::prodIncluidoNaRetirada', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }

  get maskProd() {
    return [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ];
  }

  // modalRomaneio functions
  private romaneio: IDictionary = {};

  public showModalItems = false;

  public get notShowModalItems() {
    return !this.showModalItems;
  }

  public createModal() {
    this.modalController.createModal('/modals/formRomaneio');
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }

  public addRomaneio() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    if (agregada.value) {
      this.showModalItems = false;
      this.modalController.openModal();
      // this.focusOnData();
    } else this.utilities.showAlert('errorUnidade', 'pme::unidadeNaoSelecionada', 'error');
  }

  public closeRomaneio(closeDialog = false) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    
    this.romaneio = {};
    this.itemSaved = false;
    if (closeDialog) DialogService.hide();
    this.modalController.closeModal();
    gridRomaneio.datasource.get();
    gridItems.datasource.get();
  }

  public focusOnData({ component }: IEventParam<ZdDate>) {
    const central = this.appController.getCdCentral();
    this.utilities.getProcessDate(central).then((date: string) => {
      component.value = date;
    });

    setTimeout(() => {
      this.utilities.focusOnField('DATEROMANEIO');
    }, 1);
  }

  public confirmaRomaneio() {
    const formRomaneio = Metadata.getInstance<Form>('formRomaneio');
    if (formRomaneio.validate()) {
      const agregada: Select = Metadata.getInstance('UNIDADE');
      this.romaneio = {
        CDFILIAL: agregada.value,
        DTINVENTRETI: formRomaneio.value.DATEROMANEIO,
        DSINVENTRETI: formRomaneio.value.DESCRIPTION,
        IDRETIEFET: 'N',
        NRINVENTRETI: null,
      };
      this.showModalItems = true;
      setTimeout(() => {
        this.utilities.focusOnField('PRODUCT');
      }, 1);
    }
  }

  public saveItemForm() {
    const formItems = Metadata.getInstance<Form>('formItems');
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (formItems.validate()) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: formItems.value.PRODUCT,
          QTITINVENTRETI: formItems.value.QTD,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      
      Http.post('/saveRomaneio', row).then((response) => {
        this.itemSaved = true;
        this.romaneio.NRINVENTRETI = response.data.dataset.nrseqinvent[0];
        this.utilities.showAlert('alert', 'pme::prodIncluidoNaRetirada', 'success');
      });

      product.setValue('');
      qtd.value = '';
      this.utilities.focusOnField('PRODUCT');
    }
  }

  public confirmFormRomaneio() {
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (product.value && qtd.value) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: product.value,
          QTITINVENTRETI: qtd.value,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      
      Http.post('/saveRomaneio', row).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.closeRomaneio(false);
      });
    } else if (!this.itemSaved) {
      DialogService.show({
        name: 'finalizarRomeneio',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::cancelarInclusaoRomaneio',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.closeRomaneio(true) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else this.closeRomaneio(false);
  }
}
