import {
  Metadata, Loader, Http, IEventParam, IDictionary, Config,
} from '@zeedhi/core';
import {
  GridEditable, Form, IButtonEvent, IGridEvent, ITabs, Select, Button, Tabs, TextInput, TreeGrid, Grid, DialogService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00300TipoDeCardapioController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');

  public modalController = Loader.getInstance('ModalController');

  public formTpCardapioController = Loader.getInstance('FormTpCardapioController');

  public gridEditController = Loader.getInstance('GridEditableController');

  public editable = false;
  
  public newRow = false;

  public activeArv: IDictionary = {};

  public selectedTipoCard?: string = '';

  public selectedServ?: string = '';

  public selectedNmServ?: string = '';
  
  public selectedUnidade?: string = '';

  public selectedNmUnidade?: string = '';

  private maxNrSeq = 0;
  
  public filterTipoCardapio() {
    const cardapio = Metadata.getInstance<GridEditable>('TIPOCARD');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: this.activeArv.NRPASTATCARD || null,
      },
    ];
    cardapio.datasource.filter = conditions;
    if (cardapio.editing) this.gridEditController.defaultState(cardapio);
    cardapio.datasource.get().then(() => {
      const firstRow = cardapio.datasource.data[0];
      if (firstRow) cardapio.datasource.currentRow = firstRow;
      else cardapio.datasource.currentRow = {};
      
      this.clearSelectedVal(false);
      this.filterGridServ();
      this.filterUnidadeTipoCard();
    });
  }

  public saveArvTpCard({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formTpCardapio');
    if (form.validate()) {
      const row = form.value;
      const arvPai = this.formTpCardapioController.getArvPai();
      if (this.formTpCardapioController.semPai) {
        row.NRPASTAPAI = null;
        row.NMPASTAPAI = null;
        row.NIVELPAI = null;
      } else {
        row.NRPASTAPAI = arvPai.NRPASTATCARD ? arvPai.NRPASTATCARD : null;
        row.NMPASTAPAI = arvPai.NMPASTATCARD ? arvPai.NMPASTATCARD : null;
        row.NIVELPAI = arvPai.NRNVPASTCARD ? arvPai.NRNVPASTCARD : null;
      }
      row.CDFILCENTPME = this.appController.getCdCentral();
      this.utilities.activeLoadingButton(component);
      Http.post('/saveArvTpCard', { requestType: 'Row', row }).then((response) => {
        this.formTpCardapioController.semPai = false;
        if (response) {
          const nome = Metadata.getInstance<TextInput>('NMPASTATCARD');
          nome.value = '';

          const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
          const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');

          this.editable = false;
          
          gridServico.datasource.filter = [];
          gridTpCard.datasource.filter = [];

          if (gridServico.editing) {
            this.gridEditController.defaultState(gridServico).then(() => {
              gridServico.datasource.data = [];
            });
          } else gridServico.datasource.data = [];

          if (gridTpCard.editing) {
            this.gridEditController.defaultState(gridTpCard).then(() => {
              gridTpCard.datasource.data = [];
            });
          } else gridTpCard.datasource.data = [];

          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
          this.modalController.closeModal();
          this.reloadTree();
        }
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }

  public saveTipoCard() {
    const form = Metadata.getInstance<Form>('formTipocard');
    if (form.validate()) {
      const grid = Metadata.getInstance<GridEditable>('TIPOCARD');
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.NRPASTATCARD = this.activeArv ? (this.activeArv as IDictionary).NRPASTATCARD : null;
        e.CDFILCENTPME = this.appController.getCdCentral();
      });
      this.callSave('/saveTipoCard', rows, grid);
    }
  }

  public saveServTpCard({ component }: IButtonEvent) {
    const grid = component.parent as GridEditable;
    const form: Form = Metadata.getInstance<Form>('formServico');
    if (form.validate()) {
      const rows: IDictionary = grid.getEditedRows();
      const central = this.appController.getCdCentral();
      rows.forEach((e: IDictionary) => {
        // if (e.__is_new) e.CDSERVFILI = e.NMSERVICO;
        // e.CDPRODUTO = e.NMPRODUTO || null;
        e.CDTIPOCARD = this.selectedTipoCard;
        e.CDFILCENTPME = central;
      });
      if (central === null) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'red');
      else this.callSave('/saveServTpCard', rows, grid);
    }
  }

  public saveCusto({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formCustoPadr');
    if (form.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = this.selectedServ;
        e.CDTIPOCARD = this.selectedTipoCard;
      });

      this.callSave('/saveCusto', rows, grid);
    }
  }

  public saveConfigPrat({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formConfig');
    if (form.validate()) {
      const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
      let rows: IDictionary = grid.getEditedRows();
      const addRows = grid.datasource.data.filter((e: IDictionary) => e.__is_new);
      rows = rows.concat(addRows);

      const filteredRows = rows.map((row: IDictionary) => row.NRORDCNFTCAR) // Usa uma key unica do objeto para utilizar na comparação
        .map((cdprato: string, index: number, final: string) => final.indexOf(cdprato) === index && index) // Pega o index de cada elemento do array.
        .filter((num: number) => rows[num])
        .map((position: number) => rows[position]);
      
      filteredRows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = this.selectedServ;
        e.CDTIPOCARD = this.selectedTipoCard;
        // e.NRSEQCNFTCAR = e.NRORDCNFTCAR;
      });

      this.callSave('/saveConfigPrat', filteredRows, grid);
    }
  }

  private uniqueValues(rows: IDictionary): IDictionary {
    const filteredRows: IDictionary = [];
    let validRow;
    rows.forEach((e: IDictionary) => {
      validRow = true;
      filteredRows.forEach((item: IDictionary) => {
        if (e.CDTIPOPRAT === item.CDTIPOPRAT && e.NRORDCNFTCAR === item.NRORDCNFTCAR) validRow = false;
      });
      if (validRow) filteredRows.push(e);
    });

    return filteredRows;
  }

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.editable = false;
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        this.gridEditController.defaultState(grid).then(() => {
          if (grid.name === 'SERVTIPOCARD') {
            const firstRow = grid.datasource.data[0];
            if (firstRow) grid.datasource.currentRow = firstRow;
          }
        });
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }

  public addArvTipoCard() {
    this.modalController.openModal();
    this.formTpCardapioController.setArvPai(this.activeArv);
    if (!this.activeArv.NRPASTATCARD) this.formTpCardapioController.semPai = true;
  }
  
  public closeArvTipoCard() {
    this.modalController.closeModal();
  }

  public addRow({ component }: IDictionary) {
    this.editable = true;
    let grid: GridEditable;
    
    if (component.type === 'button') grid = component.parent as GridEditable;
    else grid = component;

    if (!this.activeArv.NRPASTATCARD) this.utilities.showAlert('alertError', 'pme::tipoArvCardNaoSelecionada', 'warning');
    else if (grid.name !== 'TIPOCARD' && !this.selectedTipoCard) this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    else if (!this.gridEditController.newRow) {
      const item = { IDTPCARDATIV: '', __is_new: true };
      if (grid.name === 'TIPOCARD') {
        item.IDTPCARDATIV = 'S';
        this.clearSelectedVal(true);
        const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
        const tabPai = Metadata.getInstance<ITabs>('tabPai');
        gridServico.datasource.data = [];
        
        if (tabPai.activeTab === 1) {
          const gridUnidadeTpCard = Metadata.getInstance<GridEditable>('UNIDADECARD');
          gridUnidadeTpCard.datasource.data = [];
        }
      }
      grid.datasource.data.unshift(item);
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }

  public cancelEditing({ component }: IButtonEvent|IGridEvent) {
    this.editable = false;
    this.gridEditController.cancelEditing({ component });
  }

  public setActiveArvTpCard({ row }: IGridEvent) {
    const gridTipoCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    gridTipoCard.selectedRows = [];

    this.activeArv = row;
    this.formTpCardapioController.semPai = false;
    this.filterTipoCardapio();
  }

  public setTipoCard({ row }: IGridEvent) {
    this.selectedTipoCard = row.CDTIPOCARD;
    this.clearSelectedVal(false);
    this.filterGridServ();
    this.filterUnidadeTipoCard();
  }

  private clearSelectedVal(clearTpCard = false) {
    const tabs = Metadata.getInstance<ITabs>('tabs');
    tabs.activeTab = 0;
    if (clearTpCard) this.selectedTipoCard = '';
    this.selectedServ = '';
    this.selectedNmServ = '';
  }

  public setServ({ row }: IGridEvent) {
    this.selectedServ = row.CDSERVFILI;
    this.selectedNmServ = `${row.CDSERVFILI} - ${row.NMSERVICO}`;
  }

  public filterGrids({ component }: IEventParam<Tabs>) {
    if (component.name === 'tabPai') {
      if (component.activeTab === 1) this.filterUnidadeTipoCard();
    } else {
      if (component.activeTab === 1) this.filterGridConfPrat();
      if (component.activeTab === 2) this.filterGridCust();
    }
  }

  public filterGridServ() {
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: this.selectedTipoCard,
      },
    ];
    
    gridServico.datasource.filter = conditions;
    if (gridServico.editing) this.gridEditController.defaultState(gridServico);
    gridServico.datasource.get().then(() => {
      const firstRow = gridServico.datasource.data[0];
      if (firstRow) {
        gridServico.datasource.currentRow = firstRow;
      }
    });
  }

  public filterGridConfPrat() {
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: this.selectedServ,
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: this.selectedTipoCard,
      },
    ];
    if (gridConf.editing) this.gridEditController.defaultState(gridConf);
    gridConf.datasource.filter = conditions;
    gridConf.datasource.get();
  }

  public filterGridCust() {
    const gridCust = Metadata.getInstance<GridEditable>('CUSTPADR');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: this.selectedTipoCard,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: this.selectedServ,
      },
    ];
    if (gridCust.editing) this.gridEditController.defaultState(gridCust);
    gridCust.datasource.filter = conditions;
    gridCust.datasource.get();
  }

  public filterProd({ component }: IEventParam<Select>) {
    component.datasource.get();
  }
  
  public filterServ() {
    const cdfilcentpme = this.appController.getCdCentral();

    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilcentpme,
      },
    ];
  }

  public deleteServTpCard({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;

    if (checkedRows.length === 0) this.utilities.showAlert('alertWarning', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteServ() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private callDeleteServ() {
    const grid = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const checkedRows = grid.selectedRows;
    checkedRows.forEach((e: IDictionary) => {
      e.CDTIPOCARD = this.selectedTipoCard;
    });
    Http.post('/crud/delete', { checkedRows, dataSourceName: 'servTipoCard' }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get().then(() => {
        this.setSelectedServ();
      });
      grid.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'green');
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
    });
  }

  public addArrow() {
    const gridTipoprat = Metadata.getInstance<GridEditable>('TIPOPRAT');
    const currentRow = { ...gridTipoprat.datasource.currentRow };
    const cdfilcentpme = this.appController.getCdCentral();

    this.maxNrSeq += 1;

    currentRow.NRORDCNFTCAR = this.maxNrSeq;
    currentRow.NRSEQCNFTCAR = currentRow.NRORDCNFTCAR;
    currentRow.IDIMPCNFTCAR = 'S';
    currentRow.IDOPCCNFTCAR = 'N';
    currentRow.NEW = 'S';
    currentRow.KEY = `${cdfilcentpme}${this.selectedTipoCard}${this.selectedServ}${currentRow.NRORDCNFTCAR}${currentRow.CDTIPOPRAT}${currentRow.IDOPCCNFTCAR}${currentRow.IDIMPCNFTCAR}${currentRow.NRORDCNFTCAR}`;
    this.gridEditController.addRowArrow(currentRow, 'CONFIGTIPOPRAT');
  }

  public addGroupArrow() {
    const gridTipoprat = Metadata.getInstance<GridEditable>('TIPOPRAT');
    const rows = [...gridTipoprat.datasource.data];
    const cdfilcentpme = this.appController.getCdCentral();

    rows.forEach((e: IDictionary) => {
      this.maxNrSeq += 1;
      e.NRORDCNFTCAR = this.maxNrSeq;
      e.NRSEQCNFTCAR = e.NRORDCNFTCAR;
      e.IDIMPCNFTCAR = 'S';
      e.IDOPCCNFTCAR = 'N';
      e.NEW = 'S';
      e.KEY = `${cdfilcentpme}${this.selectedTipoCard}${this.selectedServ}${e.NRORDCNFTCAR}${e.CDTIPOPRAT}${e.IDOPCCNFTCAR}${e.IDIMPCNFTCAR}${e.NRORDCNFTCAR}`;
      this.gridEditController.addRowArrow(e, 'CONFIGTIPOPRAT');
    });
  }

  public get enableTab(): boolean {
    return (this.selectedServ === '' || this.selectedServ === undefined);
  }

  public createModal() {
    this.modalController.createModal('/modals/formTpCardapio');
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }

  public setSelectedServ() {
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const firstRow = gridServico.datasource.data[0];
    if (firstRow) {
      this.selectedServ = firstRow.CDSERVFILI;
      this.selectedNmServ = `${firstRow.CDSERVFILI} - ${firstRow.NMSERVICO}`;
    } else {
      this.selectedServ = '';
      this.selectedNmServ = '';
    }
  }

  public setSelectedTpCard() {
    const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const firstRow = gridTpCard.datasource.data[0];
    if (firstRow && firstRow.CDTIPOCARD) {
      this.selectedTipoCard = firstRow.CDTIPOCARD;
    } else {
      this.selectedTipoCard = '';
    }
  }
  
  public setSelectedUnidade() {
    const gridUnidade = Metadata.getInstance<GridEditable>('UNIDADECARD');
    const firstRow = gridUnidade.datasource.data[0];
    if (firstRow) {
      this.selectedUnidade = firstRow.CDFILIAL;
      this.selectedNmUnidade = `${firstRow.CDFILIAL} - ${firstRow.NMFILIAL}`;
    } else {
      this.selectedUnidade = '';
      this.selectedNmUnidade = '';
    }
  }

  public filterUnidadeTipoCard() {
    const tabPai = Metadata.getInstance<ITabs>('tabPai');
    if (tabPai.activeTab === 1) {
      const gridUnidadeTpCard = Metadata.getInstance<GridEditable>('UNIDADECARD');
      const cdfilcentpme = this.appController.getCdCentral();
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: this.selectedTipoCard,
        },
      ];
      
      gridUnidadeTpCard.datasource.filter = conditions;
      if (gridUnidadeTpCard.editing) this.gridEditController.defaultState(gridUnidadeTpCard);
      gridUnidadeTpCard.datasource.get().then(() => {
        const firstRow = gridUnidadeTpCard.datasource.data[0];
        if (firstRow) {
          gridUnidadeTpCard.datasource.currentRow = firstRow;
        }
      });
    }
  }
  
  public addRowUnidadeTipoCard({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    if (grid.name !== 'TIPOCARD' && !this.selectedTipoCard) this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    else if (!this.gridEditController.newRow) {
      const item = { IDTCARATFILI: 'S', __is_new: true };
      grid.datasource.data.unshift(item);
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }
  
  public saveUnidadeTipoCard({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formUnidadeTpCard');

    if (form.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        if (e.__is_new) {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDTIPOCARD = this.selectedTipoCard;
          e.CDFILIAL = e.CDFILIAL.CDFILIAL;
        }
      });
      this.callSave('/saveUnidadeTipoCard', rows, grid);
    }
  }
  
  public setFilterGridUnidade() {
    const grid = Metadata.getInstance<GridEditable>('gridTipoCardUnidade');
    const nrorg = localStorage.getItem('NRORG');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
    ];

    grid.datasource.filter = filter;
    if (grid.editing) this.gridEditController.defaultState(grid);
    grid.datasource.get();
  }

  public setFilterFieldUnidade({ component }: IEventParam<Select>) {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterTpCardapio() {
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
  }
  
  public addRowTipoCardUnidade({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilter');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const grid = this.utilities.returnGrid(component) as GridEditable;

    if (form.validate()) {
      if (!this.newRow) {
        if (unidade.value !== null) {
          if (!this.gridEditController.newRow) {
            const itens = { IDTCARATFILI: 'S', IDFATUPROD: 'S', __is_new: true };
            grid.datasource.data.unshift(itens);
            this.gridEditController.newRow = true;
            grid.editing = true;
            if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
          }
        }
      } else unidade.validations.required.message = 'pme::campoObrigatorio';
    }
  }
  
  public saveTipoCardUnidade({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formTipoCardUn');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');

    if (form.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        if (e.__is_new) {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDTIPOCARD = e.CDTIPOCARD.CDTIPOCARD;
          e.CDFILIAL = unidade.value;
        }
      });
      this.callSave('/saveTipoCardUnidade', rows, grid);
    }
  }
  
  public setFilterTreeGrid() {
    const treeGrid = Metadata.getInstance<TreeGrid>('TREEGRID');
    const cdfilcentpme = this.appController.getCdCentral();
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTCARANT',
        operator: '=',
        value: 'T',
      },
    ];
    treeGrid.datasource.filter = filter;
    treeGrid.datasource.filter.limit = 1000;
    treeGrid.datasource.get();
  }
  
  public loadGridTipo({ component }: IEventParam<Grid>) {
    component.datasource.get().then(() => {
      const data = component.datasource.data;
      if (data.length > 0) component.datasource.currentRow = data[0];
    });
  }

  public getMaxSeq() {
    const cdfilcentpme = this.appController.getCdCentral();
    const parametros = {
      CDFILCENTPME: cdfilcentpme,
      CDSERVFILI: this.selectedServ,
      CDTIPOCARD: this.selectedTipoCard,
    };
    Http.get('confTipoPrato', { params: parametros }).then((response: IDictionary) => {
      const rowsJaExistentes: IDictionary = response.data ? response.data.data : [];
      if (rowsJaExistentes.length > 0) {
        const nrseqExistentes = rowsJaExistentes.map((row: IDictionary) => row.NRORDCNFTCAR);
        this.maxNrSeq = Math.max(...nrseqExistentes);
      }
    });
  }

  public reloadTree() {
    const treeGrid = Metadata.getInstance<TreeGrid>('TREEGRID');
    treeGrid.buildTree();
    this.activeArv = {};
    const cardapio = Metadata.getInstance<GridEditable>('TIPOCARD');
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');

    cardapio.datasource.data = [];
    gridServico.datasource.data = [];

    this.clearSelectedVal(true);
    this.filterUnidadeTipoCard();
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    events: {
      click: this.filterProd,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };

  public deleteTree({ component }: IDictionary) {
    const treeGrid = Metadata.getInstance<TreeGrid>('TREEGRID');
    const row = treeGrid.datasource.currentRow;
    if (!row.NRPASTATCARD) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluir',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteTree() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private callDeleteTree() {
    const treeGrid = Metadata.getInstance<TreeGrid>('TREEGRID');
    const row = treeGrid.datasource.currentRow;
    Http.post('deleteTree', { requestType: 'Row', row }).then((response) => {
      if (response) {
        const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
        const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');

        this.editable = false;
        if (gridServico.editing) this.gridEditController.defaultState(gridServico);
        if (gridTpCard.editing) this.gridEditController.defaultState(gridTpCard);
        this.reloadTree();
        this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
    });
  }
  
  private realizaImportacaoConfig() {
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const tpCard = gridTpCard.datasource.currentRow.CDTIPOCARD;
    const rowsGridConf = gridConf.datasource.data;
    const cdfilial = this.appController.getCdCentral();
    const parametrosDaQueryInsert = {
      CDFILIAL: cdfilial,
      CDSERVFILI: this.selectedServ,
      CDTIPOCARD: tpCard,
    };
    Http.post('/importaConfigPadraoTpCard', { rowsGridConf, parametrosDaQueryInsert }).then(() => {
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::importadoComSucesso', 'success');
      gridConf.datasource.get();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public realizaAcaoExportacao() {
    const cdfilial = this.appController.getCdCentral();
    const parametrosDaQuery = {
      CDFILIAL: cdfilial,
      CDSERVFILI: this.selectedServ,
    };
    try {
      Http.post('/getDataConfigPadraoTpCard', { requestType: 'Row', row: parametrosDaQuery }).then((response: IDictionary) => {
        const configCadastrada: IDictionary = response.data.dataset.result ? response.data.dataset.result : [];
        if (configCadastrada.length > 0) {
          DialogService.show({
            name: 'casoPossuaConfig',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::desejaImportarConfigPadrao',
            buttons: [
              {
                name: 'yes',
                component: 'ZdButton',
                label: 'pme::sim',
                events: { click: () => this.realizaImportacaoConfig() },
                outline: true,
              },
              {
                name: 'no',
                component: 'ZdButton',
                label: 'pme::nao',
                events: { click: () => DialogService.hide() },
              },
            ],
          });
        } else {
          DialogService.show({
            name: 'casoNaoPossuaConfig',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::nenhumConfigPadrao',
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: 'pme::OK',
                events: { click: () => DialogService.hide() },
                outline: true,
              },
            ],
          });
        }
      });
    } catch (Error) {
      this.utilities.showAlert('alertError', 'pme::erroImportacaoConfiguracao', 'error');
    }
  }
}
