import {
  Http,
  IEventParam,
  Loader,
  Metadata,
  Config,
  IDictionary,
} from '@zeedhi/core';
import {
  Form, Checkbox, Button, Select, Increment,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00900RequisicaoServAnalController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public disableLinhaBranca() {
    const linhaBranco = Metadata.getInstance<Checkbox>('LINHASBRANCO');
    const numLinhaBranco = Metadata.getInstance<Increment>('NRLINHABRANCA');
    
    if (linhaBranco.value === true) numLinhaBranco.disabled = false;
    else {
      numLinhaBranco.disabled = true;
      numLinhaBranco.value = 0;
    }
  }

  public calculateCons({ component }: IEventParam<Button>) {
    const form: Form = Metadata.getInstance<Form>('FORM');
    if (this.formIsValid(form)) {
      const row = form.value;
      row.CDFILCENTPME = this.appController.getCdCentral();
      row.CDARVPRODINI = row.PRODINICIAL ? row.PRODINICIAL.CDARVPROD : null;
      row.CDARVPRODFIN = row.PRODFINAL ? row.PRODFINAL.CDARVPROD : null;
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        this.utilities.activeLoadingRoundButton(component);
        Http.post('calculate', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alertSuccess', 'pme::calculadoComSucesso', 'success');
        }).finally(() => {
          this.utilities.disableLoadingRoundButton(component);
        });
      }
    }
  }
  
  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        const prodIni = row.PRODINICIAL;
        const prodFin = row.PRODFINAL;
        if ((prodIni == null || prodFin == null) || this.utilities.prodIsValid(prodIni.CDARVPROD, prodFin.CDARVPROD)) valid = true;
        else this.utilities.showAlert('alertError', 'pme::prodIniMaior', 'error');
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
 
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    
    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      if (row.GRUPFILI != null) {
        row.CDGRUPFILI = row.GRUPFILI.CDGRUPFILI;
        row.NMGRUPFILI = row.GRUPFILI.NMGRUPFILI;
      } else {
        row.CDGRUPFILI = null;
        row.NMGRUPFILI = null;
      }
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/requisicaoAnalServ', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setValueQuebra(row: IDictionary) {
    const quebra = Metadata.getInstance<Checkbox>('QUEBRAPAG').value;
    
    if (quebra === false) row.QUEBRAPAG = false;
    
    else row.QUEBRAPAG = true;
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    this.setValueQuebra(row);
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.DTPREVCONPME = row.DTPREVCONPME ? row.DTPREVCONPME.DTPREVCONPME : null;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.TIPOCARD = row.TIPOCARD ? row.TIPOCARD : null;
    row.UNIDADE = row.UNIDADE ? row.UNIDADE : null;
    row.SERVICO = row.SERVICO ? row.SERVICO : null;
    row.NRLISTA = row.NRLISTA ? row.NRLISTA.NRLISTA : null;
    row.TAMNIV = this.utilities.calcTamacNivel(row.TAMNIV, 'produto');
    row.CDPICTPROD = this.utilities.formatReportMask(mask);
    row.CDARVPRODINI = row.PRODINICIAL ? row.PRODINICIAL.CDARVPROD : null;
    row.CDARVPRODFIN = row.PRODFINAL ? row.PRODFINAL.CDARVPROD : null;
  }
}
