import {
  Http, IDictionary, IEventParam, Loader, Metadata, Config,
} from '@zeedhi/core';
import {
  Button, Column, DialogService, Form, Grid, GridEditable, IButtonEvent, IGridEvent, Select, SelectMultiple, Tabs, TextInput,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00200ServicoController {
  private utilities = Loader.getInstance('Utilities');
  
  public crudController = Loader.getInstance('CrudController');
  
  public appController = Loader.getInstance('AppController');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');

  public editable = false;

  public activeArv: object | undefined;

  public selectedServ?: string = '';

  public selectedNmServ?: string = '';
  
  public unidade?: string = '';
  
  public checkedRowsServ: IDictionary = {};
  
  public checkedRowsTipo: IDictionary = {};
  
  public currentRowCopy: IDictionary = {};
  
  public nrseqconfcard = 0;
  
  public returnConditionsUnidade(nameFilter: string) {
    const unidSelecionada = Metadata.getInstance<Select>('UNIDADE');
    const conditions = [
      {
        name: nameFilter,
        operator: '=',
        value: unidSelecionada.value.CDFILIAL,
      },
    ];
    return conditions;
  }
  
  public fillCode({ component }: IEventParam<TextInput>) {
    component.value = component.value.padStart(5, '0');
  }
  
  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response.data.dataset.success) {
        this.editable = false;
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        this.gridEditController.defaultState(grid).then(() => {
          const tabs = Metadata.getInstance<Tabs>('tabs');
          const tabConfig = tabs.tabs[1];
          tabConfig.disabled = false;
          const firstRow = grid.datasource.data[0];
          if (firstRow) {
            grid.datasource.currentRow = firstRow;
            this.setServ(grid.datasource.currentRow);
          }
        });
      }
    }).catch(() => {
      this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
    });
  }
  
  public addRow({ component }: IButtonEvent) {
    this.editable = true;
    const grid = component.parent as GridEditable;

    if (!this.gridEditController.newRow) {
      const item = { __is_new: true };
      grid.datasource.data.unshift(item);
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }

  public cancelEditing({ component }: IButtonEvent) {
    this.editable = false;
    this.gridEditController.cancelEditing({ component });
  }
  
  public verificaPadraoServ({ component }: IEventParam<GridEditable>) {
    Http.post('/verificaPadraoServ', 'Row').then((response) => {
      const nmServInput = this.returnColumn(component, 'NMSERVICOINPUT') as Column;
      const nmServSelect = this.returnColumn(component, 'NMSERVICO') as Column;
      if (response.data.dataset.result.length > 0) {
        nmServInput.isVisible = false;
        nmServSelect.isVisible = true;
      } else {
        nmServInput.isVisible = true;
        nmServSelect.isVisible = false;
      }
    });
  }
  
  public returnColumn(component: GridEditable, nmColuna: string) {
    return component.columns.find((column) => column.name === nmColuna);
  }
  
  public clickRowGrid({ row }: IGridEvent) {
    this.setServ(row);
  }
  
  public filterGrids({ component }: IEventParam<Tabs>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    if (component.activeTab === 1) {
      unidade.disabled = true;
      this.filterGridConfPrat();
    } else unidade.disabled = false;
  }
  
  public loadGridTipo({ component }: IEventParam<Grid>) {
    component.datasource.get().then(() => {
      const data = component.datasource.data;
      if (data.length > 0) component.datasource.currentRow = data[0];
      this.getMaxSeq();
    });
  }
  
  // Filtro
  public setFilterGridService({ component }: IEventParam<Select>) {
    const gridServ = Metadata.getInstance<GridEditable>('SERVICOS');
    const cdfilial = component.value;
    const addButton = Metadata.getInstance<Button>('addButton');
    const exportButton = Metadata.getInstance<Button>('exportButton');
    const tabs = Metadata.getInstance<Tabs>('tabs');
    const tabConfig = tabs.tabs[1];
    if ((cdfilial === null) || (cdfilial && cdfilial.IDFILCENTPME !== 'S' && cdfilial.CENTRAL === null)) {
      if (cdfilial && cdfilial.IDFILCENTPME !== 'S' && cdfilial.CENTRAL === null) this.utilities.showAlert('alertError', 'pme::unidadeSemCentralMerenda', 'error');
      gridServ.datasource.filter = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: '',
        },
        {
          name: 'P_NRORG',
          operator: '=',
          value: '',
        },
      ];
      gridServ.datasource.get().then(() => {
        exportButton.disabled = true;
        addButton.disabled = true;
        tabConfig.disabled = true;
      });
    } else {
      gridServ.datasource.filter = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: cdfilial.CDFILIAL,
        },
        {
          name: 'P_NRORG',
          operator: '=',
          value: localStorage.getItem('NRORG'),
        },
      ];
      gridServ.datasource.get().then(() => {
        if (cdfilial.CDFILIAL) {
          addButton.disabled = false;
          const data = gridServ.datasource.data;
          if (data.length > 0) {
            tabConfig.disabled = false;
            gridServ.datasource.currentRow = data[0];
            this.setServ(gridServ.datasource.currentRow);
          } else {
            tabConfig.disabled = true;
          }
          if (cdfilial.IDFILCENTPME === 'S' && cdfilial.IDTPFILIALCENT === 'M') {
            exportButton.disabled = false;
            this.setDataModalExport(component);
          } else {
            exportButton.disabled = true;
          }
        } else {
          addButton.disabled = true;
          exportButton.disabled = true;
        }
      });
    }
  }
  
  public setDataModalExport(component: Select) {
    this.unidade = component.formatter(component.value);
    const unidAgregada = Metadata.getInstance<Select>('UNIDADEAGREG');
    const serviceExport = Metadata.getInstance<Select>('SERVICO_EXPORT');
    unidAgregada.datasource.filter = this.returnConditionsUnidade('CDFILCENTPME');
    unidAgregada.datasource.get();
    serviceExport.datasource.filter = this.returnConditionsUnidade('CDFILIAL');
    serviceExport.datasource.get();
  }
  
  // Aba de Serviços(Grid)
  public filterServ({ component }: IEventParam<Select>) {
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilcentpme,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public filterProd({ component }: IEventParam<Select>) {
    component.datasource.get();
  }
  
  public saveService({ component }: IButtonEvent) {
    const form = Metadata.getInstance<Form>('formServico');
    if (form.validate()) {
      const grid = component.parent as GridEditable;
      const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
      const rows: IDictionary = grid.getEditedRows();
      let rowsInvalid = false;
      rows.forEach((e: IDictionary) => {
        if (this.validateFieldsHour(e.HRINICSERV, e.HRFINASERV)) {
          e.CDSERVFILI = e.CDSERVFILI.padStart(5, '0');
          e.HRINICATEN = e.HRINICATEN ? this.formatHrToSave(e.HRINICATEN) : null;
          e.HRINICSERV = e.HRINICSERV && e.HRFINASERV ? this.formatHrToSave(e.HRINICSERV) : null;
          e.HRFINASERV = e.HRFINASERV && e.HRINICSERV ? this.formatHrToSave(e.HRFINASERV) : null;
          e.CDGRUPSERV = e.CDGRUPSERV ? e.CDGRUPSERV : null;
          e.NMSERVICO = e.NMSERVICO ? e.NMSERVICO : e.NMSERVICOINPUT;
          e.CDPRODUTO = e.CDPRODUTO ? e.CDPRODUTO : e.PRODUTO.CDPRODUTO;
          e.CDFILIAL = cdfilial;
        } else {
          rowsInvalid = true;
          // break;
        }
      });
      if (rowsInvalid) this.utilities.showAlert('alertError', 'pme::horaInicialMaiorFinal', 'error');
      else this.callSave('/saveServ', rows, grid);
    }
  }
  
  public validateFieldsHour(hourIni: string, hourFin: string) {
    if (hourIni && hourIni !== '' && hourFin && hourFin !== '') {
      const hrIni = hourIni.split(':');
      const secondsIni = (+hrIni[0]) * 60 * 60 + (+hrIni[1]) * 60;
      const hrFin = hourFin.split(':');
      const secondsFin = (+hrFin[0]) * 60 * 60 + (+hrFin[1]) * 60;
      
      if (secondsIni < secondsFin) return true;
      return false;
    }
    return true;
  }
  
  public formatHrToSave(hour: string) {
    return hour.replace(':', '');
  }
  
  public confirmDeleteServ({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    this.checkedRowsServ = grid.selectedRows;
    if (this.checkedRowsServ.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pma00200_excluiServico',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteServ(this.checkedRowsServ, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteServ(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/deleteServ', { checkedRows }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get().then(() => {
        if (grid.datasource.data.length === 0) {
          const tabs = Metadata.getInstance<Tabs>('tabs');
          const tabConfig = tabs.tabs[1];
          tabConfig.disabled = true;
        }
      });
      grid.selectedRows = [];
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  // Aba de Configuração
  
  public confirmDeleteConfig({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    this.checkedRowsTipo = grid.selectedRows;
    if (this.checkedRowsTipo.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pme00200_excluiConfiguracaoServ',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteRowsConfigPrat(this.checkedRowsTipo, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteRowsConfigPrat(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/deleteConfig', { checkedRows }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get();
      grid.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'green');
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public setServ(row: IDictionary) {
    if (row) {
      this.selectedServ = row.CDSERVFILI;
      this.selectedNmServ = `${row.CDSERVFILI} - ${row.NMSERVICO}`;
    } else {
      this.selectedServ = '';
      this.selectedNmServ = '';
    }
  }

  public filterGridConfPrat() {
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: this.selectedServ,
      },
    ];
    if (gridConf.editing) this.gridEditController.defaultState(gridConf);
    gridConf.datasource.filter = conditions;
    gridConf.datasource.get();
  }
  
  private getMaxOrdem(): number {
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    const array = gridConf.datasource.data.filter((row) => row.__is_new === false);
    const length = array.length - 1;
    const ordem = gridConf.datasource.data[length] ? gridConf.datasource.data[length].NRORDCNFCARD : 0;
    return ordem;
  }
  
  /* Pega o maior valor de NRSEQCNFCARD do banco para o serviço e aunidade selecionada
    Para não gerar KEY repetida e diferenciar as linhas do grid na hora de
    adicionar no grid de Configuração Padrão,pois a chave primária da tabela só
    é inserida na hora de salvar recebendo o número da ordem que foi digitado;
  */
  public getMaxSeq() {
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
    const parametros = {
      CDFILIAL: cdfilial,
      CDSERVFILI: this.selectedServ,
    };
    Http.post('dadosGridConfigPrat', { requestType: 'Row', row: parametros }).then((response) => {
      const rowsJaExistentes: IDictionary = response.data ? response.data.dataset.tipoprat : [];
      if (rowsJaExistentes.length > 0) {
        const nrseqExistentes = rowsJaExistentes.map((row: IDictionary) => row.NRSEQCNFCARD);
        this.nrseqconfcard = Math.max(...nrseqExistentes);
      }
    });
  }
  
  public addArrow() {
    const gridTipoprat = Metadata.getInstance<GridEditable>('TIPOPRAT');
    this.nrseqconfcard += 1;
    this.currentRowCopy = { ...gridTipoprat.datasource.currentRow };
    this.currentRowCopy.NRORDCNFCARD = this.nrseqconfcard;
    this.currentRowCopy.IDOPCCNFCARD = 'N';
    this.currentRowCopy.IDIMPCNFCARD = 'S';
    this.currentRowCopy.NEW = 'S';
    this.currentRowCopy.KEY = `${this.currentRowCopy.CDTIPOPRAT}${this.currentRowCopy.NRSEQCNFCARD}`;
    if (this.currentRowCopy.IDIMPCNFCARD === 'N') this.currentRowCopy.NRPAGCNFCARD = 0;
    this.gridEditController.addRowArrow(this.currentRowCopy, 'CONFIGTIPOPRAT');
  }
  
  public addGroupArrow() {
    const gridTipoprat = Metadata.getInstance<GridEditable>('TIPOPRAT');
    const rows = [...gridTipoprat.datasource.data];
    this.getMaxSeq();
    rows.forEach((e: IDictionary) => {
      this.nrseqconfcard += 1;
      e.NRORDCNFCARD = this.nrseqconfcard;
      e.KEY = `${e.CDTIPOPRAT}${this.nrseqconfcard}`;
      e.IDIMPCNFCARD = 'S';
      e.IDOPCCNFCARD = 'N';
      e.NEW = 'S';
      this.gridEditController.addRowArrow(e, 'CONFIGTIPOPRAT');
    });
  }
  
  /* Salva grid de configuração de prato;
    Sempre irá deletar todas as rows e inserir novamente(mesmo em caso de edição),
    pois precisa fazer o controle do parâmetro de paginação;
    Regra para salvar o NRPAGCNFCARD(paginação): A cada duas inserções onde a coluna IDIMPCNFCARD for igual a 'S'
    aumenta a paginação em 1.
  */
  public saveConfigPrat({ component }: IButtonEvent) {
    const form = Metadata.getInstance<Form>('formConfig');
    if (form.validate()) {
      const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
      const grid: GridEditable = component.parent as GridEditable;
      let rowsEdited: IDictionary = grid.getEditedRows();
      const addRows = grid.datasource.data.filter((e: IDictionary) => e.__is_new);
      rowsEdited = rowsEdited.concat(addRows);
      const parametros = {
        CDFILIAL: cdfilial,
        CDSERVFILI: this.selectedServ,
      };
      Http.post('dadosGridConfigPrat', { requestType: 'Row', row: parametros }).then((response) => {
        const rowsJaExistentes: IDictionary = response.data ? response.data.dataset.tipoprat : [];
        const rowsTotal = rowsEdited.concat(rowsJaExistentes);
        const uniqueData = rowsTotal.map((row: IDictionary) => row.NRORDCNFCARD) // Usa uma key unica do objeto para utilizar na comparação
          .map((cdprato: string, index: number, final: string) => final.indexOf(cdprato) === index && index) // Pega o index de cada elemento do array.
          .filter((num: number) => rowsTotal[num])
          .map((position: number) => rowsTotal[position]);
        let inRow = 1;
        let inPag = 1;
        uniqueData.forEach((e: IDictionary) => {
          if (e.IDIMPCNFCARD === 'N') {
            e.NRPAGCNFCARD = 0;
          } else if (e.IDIMPCNFCARD === 'S') {
            if (inRow === 1) {
              e.NRPAGCNFCARD = inPag;
              inRow = 2;
            } else {
              e.NRPAGCNFCARD = inPag;
              inRow = 1;
              inPag += 1;
            }
          }
          e.CDFILIAL = cdfilial;
          e.CDSERVFILI = this.selectedServ;
          e.NRORDCNFCARD = parseInt(e.NRORDCNFCARD, 10);
          e.NRSEQCNFCARD = e.NRORDCNFCARD;
        });
        Http.post('/saveConfigPratServ', { requestType: 'Row', row: uniqueData }).then((callback) => {
          if (callback) {
            this.editable = false;
            this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
            this.gridEditController.defaultState(grid);
          }
        }).catch(() => {
          this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
        });
      });
    }
  }
  
  // Exportar Serviços
  public openExportModal() {
    this.modalController.openModal();
  }
  
  public createModal() {
    this.modalController.createModal('/modals/formExportService');
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }
  
  public setFilterFieldsExport({ component }: IEventParam<SelectMultiple>) {
    let filterName = 'CDFILCENTPME';
    if (component.name === 'SERVICO_EXPORT') filterName = 'CDFILIAL';
    component.datasource.filter = this.returnConditionsUnidade(filterName);
    // component.datasource.get();
  }
  
  public exportService({ component }: IEventParam<Button>) {
    const formExport = Metadata.getInstance<Form>('formExportService');
    if (formExport.validate()) {
      const row = {
        UNIDADE_EXPORT: Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL,
        UNIDADEAGREG: formExport.value.UNIDADEAGREG,
        SERVICO_EXPORT: formExport.value.SERVICO_EXPORT,
      };
      row.UNIDADEAGREG = row.UNIDADEAGREG.length > 0 ? row.UNIDADEAGREG.join('_') : 'T';
      row.SERVICO_EXPORT = row.SERVICO_EXPORT.length > 0 ? row.SERVICO_EXPORT.join('_') : 'T';
      this.utilities.activeLoadingButton(component);
      Http.post('/exportService', { requestType: 'Row', row }).then((response) => {
        if (response.data.dataset.success) {
          this.utilities.showAlert('alertSuccess', 'pme::exportadoComSucesso', 'green');
          this.modalController.closeModal();
        } else {
          this.utilities.showAlert('alertError', 'pme::erroExportacao', 'red');
        }
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
