import {
  Loader, Metadata, IEventParam, Http, IDictionary, Config,
} from '@zeedhi/core';
import {
  Select, moment, Form, GridEditable, IDate, TextInput, Button, Grid, IGridEvent, ITabs,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme00700ConsPorServicoController {
  public appController = Loader.getInstance('AppController');

  public gridEditController = Loader.getInstance('GridEditableController');

  public utilities = Loader.getInstance('Utilities');

  public modalController = Loader.getInstance('ModalController');

  private tipoCons: string|null = null;

  public setFilterServ({ component }: IEventParam<Select>) {
    const unidade: Select = Metadata.getInstance('UNIDADE');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value || null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterGrids() {
    const tabs = Metadata.getInstance<ITabs>('tabs');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const unidade = formFiltro.value.UNIDADE;
    const servico = formFiltro.value.SERVICO;
    
    if (unidade === null || servico === null) this.disableAddButtons(tabs.activeTab, true);
    else this.disableAddButtons(tabs.activeTab, false);
    
    if (tabs.activeTab === 0) {
      const dtValidade = Metadata.getInstance<IDate>('DTVALIDADE');
      if (formFiltro.validate()) {
        const filter = [
          {
            name: 'CDFILCENTPME',
            operator: '=',
            value: this.appController.getCdCentral(),
          },
          {
            name: 'CDFILIAL',
            operator: '=',
            value: unidade || null,
          },
          {
            name: 'CDSERVICO',
            operator: '=',
            value: servico || null,
          },
          {
            name: 'DATAATUAL',
            operator: '=',
            value: moment().format('DD/MM/YYYY'),
          },
        ];
        
        Http.post('find/vigenciaAtualConsumo', { requestType: 'FilterData', filter }).then((response) => {
          let date = '';
          if (response.data.data.length > 0) {
            date = response.data.data[0].DTCONSERVPME;
          }
    
          dtValidade.value = date;
          
          this.setFilterGridCons();
        });
      } else {
        const gridCons = Metadata.getInstance<GridEditable>('CONSPADRAO');
        gridCons.datasource.data = [];
        dtValidade.value = '';
      }
    } else if (tabs.activeTab === 1) this.setFilterGridExcecao();
    else this.setFilterGridSemCons();
  }
  
  public disableAddButtons(activeTab: number | undefined, disable: boolean) {
    if (activeTab === 0) {
      const addButtonCons = Metadata.getInstance<Button>('addButtonCons');
      addButtonCons.disabled = disable;
    } else if (activeTab === 1) {
      const addButtonExc = Metadata.getInstance<Button>('addButtonExcecao');
      addButtonExc.disabled = disable;
    } else {
      const addButtonSemCons = Metadata.getInstance<Button>('addButtonSemCons');
      addButtonSemCons.disabled = disable;
    }
  }

  public setFilterGridCons() {
    const addButton = Metadata.getInstance<Button>('addButtonCons');
    const gridCons = Metadata.getInstance<GridEditable>('CONSPADRAO');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    const filterCons = this.getCommonFilter();
    const formCons = Metadata.getInstance<Form>('formCons');

    let date = formCons.value.DTVALIDADE;
    if (date == null || date.length < 10) date = null;

    filterCons.push(
      {
        name: 'DTCONSERVPME',
        operator: '=',
        value: date,
      },
      {
        name: 'IDTIPOCONPME',
        operator: '=',
        value: formCons.value.DIASEMANACONSUMO || null,
      },
      {
        name: 'IDQTDECONPME',
        operator: '=',
        value: formCons.value.TPCONSUMO || null,
      },
    );
    
    addButton.disabled = true;

    gridCons.datasource.filter = filterCons;
    gridCons.datasource.get().then(() => {
      if (formCons.validate()) addButton.disabled = false;
    });
    this.getComensais(unidade.value, servico.value);
  }

  public setFilterGridExcecao() {
    const addButton = Metadata.getInstance<Button>('addButtonExcecao');
    const formExcecao = Metadata.getInstance<Form>('formExcecao');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const gridExcecao = Metadata.getInstance<GridEditable>('GRIDEXCECAO');
    
    if (formFiltro.validate()) {
      const filter = this.getCommonFilter();
      
      let date = formExcecao.value.DTEXCECAO;
      if (date == null || date.length < 10) date = null;

      filter.push(
        {
          name: 'DTCONSERVPME',
          operator: '=',
          value: date,
        },
        {
          name: 'IDTIPOCONPME',
          operator: '=',
          value: '9',
        },
        {
          name: 'IDQTDECONPME',
          operator: '=',
          value: formExcecao.value.TPCONSUMOEXCECAO || null,
        },
      );
      
      addButton.disabled = true;

      gridExcecao.datasource.filter = filter;
      gridExcecao.datasource.get().then(() => {
        if (formExcecao.validate()) addButton.disabled = false;
      });
    } else {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      gridExcecao.datasource.data = [];
      dtExcecao.value = '';
    }
  }

  public setFilterGridSemCons() {
    const addButton = Metadata.getInstance<Button>('addButtonSemCons');
    const gridSemCons = Metadata.getInstance<GridEditable>('GRIDSEMCONS');
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    addButton.disabled = true;

    if (formFiltro.validate() && gridSemCons) {
      const filter = this.getCommonFilter();
      
      gridSemCons.datasource.filter = filter;
      gridSemCons.datasource.get().then(() => {
        if (formFiltro.value.UNIDADE && formFiltro.value.SERVICO) addButton.disabled = false;
      });
    } else gridSemCons.datasource.data = [];
  }

  private getCommonFilter() {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: formFiltro.value.UNIDADE || null,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: formFiltro.value.SERVICO || null,
      },
    ];
  }

  private getComensais(cdUnidade = '', cdServ = '') {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdUnidade,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdServ,
      },
    ];

    Http.post('find/consServCard', { requestType: 'FilterData', filter }).then((response) => {
      if (response.data.data.length > 0) {
        const data = response.data.data[0];
        const qtCons = Metadata.getInstance<TextInput>('QTCONSERVPME');
        const day = Metadata.getInstance<Select>('DIASEMANACONSUMO');
        let comensais = 0;

        switch (day.value) {
          case '1': comensais = data.QTCOMPMEDOM || 0; break;
          case '2': comensais = data.QTCOMPMESEG || 0; break;
          case '3': comensais = data.QTCOMPMETER || 0; break;
          case '4': comensais = data.QTCOMPMEQUA || 0; break;
          case '5': comensais = data.QTCOMPMEQUI || 0; break;
          case '6': comensais = data.QTCOMPMESEX || 0; break;
          case '7': comensais = data.QTCOMPMESAB || 0; break;
          case '8': comensais = data.QTCOMPMEFER || 0; break;
          default: comensais = 0; break;
        }
        qtCons.value = comensais;
      }
    });
  }

  public setFilterValidade({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value || null,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: servico.value || null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setVigencia({ row }: IGridEvent) {
    if (row) {
      let field!: IDate;
      if (this.tipoCons === null) field = Metadata.getInstance<IDate>('DTVALIDADE');
      else field = Metadata.getInstance<IDate>('DTEXCECAO');
      field.value = row.DTCONSERVPME;
      
      if (field.name === 'DTVALIDADE') this.setFilterGridCons();
      else this.setFilterGridExcecao();

      this.modalController.closeModal();
    }
  }

  public clearServico() {
    const servico: Select = Metadata.getInstance('SERVICO');
    servico.setValue('');
    const tabs = Metadata.getInstance<ITabs>('tabs');
    if (tabs.activeTab === 0) {
      const dtValidade = Metadata.getInstance<IDate>('DTVALIDADE');
      const qtCons = Metadata.getInstance<TextInput>('QTCONSERVPME');
      dtValidade.value = '';
      qtCons.value = '';
    } else if (tabs.activeTab === 1) {
      const dtExcecao = Metadata.getInstance<IDate>('DTEXCECAO');
      dtExcecao.value = '';
    }
  }

  public saveConsPadr({ component }: IEventParam<Button>) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formCons = Metadata.getInstance<Form>('formCons');
    
    if (formFiltro.validate() && formCons.validate()) {
      const grid = component.parent as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;
        e.DTCONSERVPME = formCons.value.DTVALIDADE;
        e.IDTIPOCONPME = formCons.value.DIASEMANACONSUMO;
        e.IDQTDECONPME = formCons.value.TPCONSUMO;
      });
      this.callSave('/saveConsPadr', rows, grid);
    }
  }

  public saveExcecao({ component }: IEventParam<Button>) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formExcecao = Metadata.getInstance<Form>('formExcecao');
    
    if (formFiltro.validate() && formExcecao.validate()) {
      const grid = component.parent as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;
        e.DTCONSERVPME = formExcecao.value.DTEXCECAO;
        e.IDTIPOCONPME = '9';
        e.IDQTDECONPME = formExcecao.value.TPCONSUMOEXCECAO;
      });
      this.callSave('/saveConsPadr', rows, grid);
    }
  }

  public saveSemCons({ component }: IEventParam<Button>) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formSemCons = Metadata.getInstance<Form>('formSemCons');
    
    if (formFiltro.validate() && formSemCons.validate()) {
      const grid = component.parent as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      let isValidRows = true;
      rows.forEach((e: IDictionary) => {
        e.CDPRODUTO = e.NMPRODUTO;
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDSERVFILI = formFiltro.value.SERVICO;
        e.CDFILIAL = formFiltro.value.UNIDADE;

        if (e.PERCAPITA === 'N' && e.FIXO === 'N') isValidRows = false;
      });

      if (isValidRows) this.callSave('/saveSemCons', rows, grid);
      else this.utilities.showAlert('alertSuccess', 'Pelo menos um dos tipos deve ser Sim.', 'warning');
    }
  }

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    });
  }

  public setFilterVigencia() {
    const gridVigencia = Metadata.getInstance<Grid>('gridVigencia');
    if (gridVigencia) {
      const unidade = Metadata.getInstance<Select>('UNIDADE');
      const servico = Metadata.getInstance<Select>('SERVICO');
      const filter = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade.value || null,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: servico.value || null,
        },
        {
          name: 'IDTIPOCONPME',
          operator: '=',
          value: this.tipoCons,
        },
      ];

      gridVigencia.datasource.filter = filter;
      gridVigencia.datasource.get();
    }
  }

  public destroyVigenciaModal() {
    this.modalController.destroyModal();
  }

  public openVigenciaModal({ component }: IEventParam<Button>) {
    const modal = this.modalController.getModal();
    if (component.name === 'searchValidade') this.tipoCons = null;
    else this.tipoCons = '9';
    if (!modal) {
      this.modalController.createModal('modals/gridVigencias').then(() => {
        this.modalController.openModal();
        this.setFilterVigencia();
      });
    } else {
      this.modalController.openModal();
      this.setFilterVigencia();
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
